<?php
/**
 * Functions for debugging
 * @since 2.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

function bfwc_calendar_page() {

	add_submenu_page(
		'edit.php?post_type=bfwc_booking',
		__( 'Calendar', 'pewc' ),
		__( 'Calendar', 'pewc' ),
		apply_filters( 'bfwc_calendar_page_capability', 'manage_options' ),
		'bfwc_calendar_page',
		'bfwc_calendar_page_callback'
	);

}
add_action( 'admin_menu', 'bfwc_calendar_page', 100 );

function bfwc_calendar_page_callback() { ?>

	<div class="wrap">
		<?php printf( '<h1>%s</h1>', __( 'Calendar', 'bfwc' ) ); ?>
		<p><?php _e( 'View availability for all your bookable products on one calendar.', 'bfwc' ); ?></p>

		<div class="bfwc-datepicker-wrapper">

			<div>
				<div id="bfwc-global-datepicker" class="bfwc-datepicker"></div>

				<table class="bfwc-availability-key">
					<tr>
						<td class="availability-key full-availability">&nbsp;<?php _e( 'Full availability', 'bfwc' ); ?></td>
						<td class="availability-key part-availability">&nbsp;<?php _e( 'Part availability', 'bfwc' ); ?></td>
						<td class="availability-key no-availability">&nbsp;<?php _e( 'No availability', 'bfwc' ); ?></td>
					</tr>
				</table>
			</div>

			<div id="bfwc-calendar-modal">
				<?php printf(
					'<p><span class="dashicons dashicons-info-outline"></span>&nbsp;%s</p>',
					__( 'Click on a date on the calendar to view additional information', 'bfwc' )
				); ?>
			</div>
			<input type="hidden" id="bfwc_selected_date" name="bfwc_selected_date">
			<?php wp_nonce_field( 'bfwc_calendar_modal_nonce', 'bfwc_calendar_modal_nonce' ); ?>

		</div>

	</div>

<?php }

/**
 * Get data for selected date in admin
 * Used in product pages
 * @since 2.0.0
 */
function bfwc_get_booking_data() {

	// Check nonce and permissions
	if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_calendar_modal_nonce' ) ) {
		wp_send_json( array( 'result' => 'nonce fail' ) );
		exit;
	}

	$date = ! empty( $_POST['date'] ) ? $_POST['date'] : false;
	$date = bfwc_get_date_from_locale( $date );

	// Format date
	$metadata = bfwc_get_booked_day_metadata( bfwc_get_ymd ( $date ) ); // 2.0.6, force convert using bfwc_get_ymd because d/m/Y sometimes causes issues if day is > 12
	$product_id = ! empty( $_POST['product_id'] ) ? $_POST['product_id'] : false;

	// Return the date in the site date format
	$date_format = get_option( 'date_format' ); // no longer in use since 1.4.4
	$formatted_date = date( $date_format, strtotime( bfwc_get_ymd( $date ) ) ); // 2.0.6, added bfwc_get_ymd(), see note above

	$booking_data = array();

	// Availability, e.g. This date is currently available to book
	// if( ! isset( $metadata['booked_meta'][$product_id]['used_bookings'] ) ) {
	if( ! isset( $metadata['booking_meta'][$product_id]['used_bookings'] ) ) {

		// Full availability
		$availability = __( 'Fully available', 'bfwc' );

	} else {

		$max = bfwc_get_max_bookings( $product_id );
		$used = $metadata['booking_meta'][$product_id]['used_bookings'];

		if( $used >= $max ) {

			$availability = __( 'No availability', 'bfwc' );

		} else {

			$remaining = $max - $used;
			$availability = sprintf(
				__( 'Partial availability: %d remaining', 'bfwc' ),
				$remaining
			);

		}

		// Get bookings for this date
		$booking_ids = ! empty( $metadata['booking_meta'][$product_id]['booking_ids'] ) ? $metadata['booking_meta'][$product_id]['booking_ids'] : false;
		if( $booking_ids ) {

			foreach( $booking_ids as $id ) {

				// Only published bookings
				// if( get_post_status( $id ) != 'publish' ) {
				// 	continue;
				// }

				$booking_url = add_query_arg(
					array(
						'post'		=> $id,
						'post_type'	=> 'bfwc_booking',
						'action'	=> 'edit'
					),
					admin_url( 'post.php' )
				);

				$args = array(
					'id'				=> $id,
					'booking_url'		=> $booking_url,
					'start'				=> bfwc_format_date( get_post_meta( $id, 'bfwc_start_date', true ) ),
					'end'				=> bfwc_format_date( get_post_meta( $id, 'bfwc_end_date', true ) )
				);

				// 2.0.5
				$order_id	= get_post_meta( $id, 'bfwc_order_id', true );
				if ( $order_id ) {
					$args['order_id'] = $order_id;
					$order = wc_get_order( $order_id );
					if ( $order ) {
						if ( bfwc_hpos_enabled() ) {
							$order_url = $order->get_edit_order_url(); // WC HPOS compatible
						} else {
							$order_url = add_query_arg(
								array(
									'post_type'		=> 'shop_order',
									'post'				=> $order_id,
									'action'			=> 'edit'
								),
								admin_url( 'edit.php' )
							);
						}

						$args['order_url'] = $order_url;
					} else {
						$args['order_url'] = '';
					}
				} else {
					$args['order_id'] = $args['order_url'] = '';
				}

				$user_id = get_post_meta( $id, 'bfwc_user_id', true );

				if( $user_id ) {
					$user = get_user_by( 'id', $user_id );
					if ( bfwc_hpos_enabled() ) {
						$user_url = add_query_arg(
							array(
								'page'				=> 'wc-orders',
								'_customer_user'	=> $user_id
							),
							admin_url( 'admin.php' )
						);
					} else {
						$user_url = add_query_arg(
							array(
								'post_type'			=> 'shop_order',
								'_customer_user'	=> $user_id
							),
							admin_url( 'edit.php' )
						);
					}
					$args['user_id']	= $user_id;
					$args['user_name']	= apply_filters( 'bfwc_user_nicename', $user->user_nicename );
					$args['user_url']	= $user_url;

				} else if ( isset( $order ) ) {
					// 2.0.5, maybe a guest user
					$args['user_name'] = trim( $order->get_billing_first_name().' '.$order->get_billing_last_name() );
					$args['user_url']  = '';
				} else {
					$args['user_name'] = '';
					$args['user_url']  = ''; 
				}

				$booking_data[] = $args;

			}

		}

	}

	$blocked_by = '';
	if( ! empty( $metadata['blocked_meta'][$product_id]['blocked_by'] ) ) {

		// This date is blocked out by an external service

		$availability .= '&nbsp;';
		$availability .= sprintf(
			__( 'Blocked by %s', 'bfwc' ),
			$metadata['blocked_meta'][$product_id]['blocked_by']
		);

	}

	// Blocked by

	wp_send_json( array(
		'date' 							=> $formatted_date,
		'availability'					=> $availability,
		'booking_data'					=> $booking_data
	) );

}
add_action( 'wp_ajax_bfwc_get_booking_data', 'bfwc_get_booking_data' );

/**
 * Get global data for selected date in admin
 * Returns a list of all bookings for this date
 * @since 2.0.0
 */
function bfwc_get_global_booking_data() {

	// Check nonce and permissions
	if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_calendar_modal_nonce' ) ) {
		wp_send_json( array( 'result' => 'nonce fail' ) );
		exit;
	}

	$date = $_POST['date'] ? $_POST['date'] : false;

	// Format date
	$metadata = bfwc_get_booked_day_metadata( bfwc_get_ymd( $date ) ); // 2.0.6, force convert using bfwc_get_ymd because d/m/Y sometimes causes issues if day is > 12

	// Return the date in the site date format
	$date_format = get_option( 'date_format' ); // no longer in use since 1.4.4
	$formatted_date = date( $date_format, strtotime( bfwc_get_ymd ( $date ) ) ); // 2.0.6, added bfwc_get_ymd(), see note above

	$booked_day_id = 0;
	if ( get_option( 'bfwc_view_booked_days', 'no' ) == 'yes' ) {
		$booked_day = bfwc_get_page_by_title( date( 'Y-m-d', strtotime( $date ) ), 'bfwc_booked_days' );
		if ( $booked_day ) {
			$booked_day_id = $booked_day->ID;
		}
	}

	$available_products = bfwc_get_all_bookable_products();

	// Use this to store a list of each product's availability on this date
	$global_availability = array();
	// Use this to list any active bookings for this date
	$bookings = array();
	$booking_data = array();
	$used_bookings = array();

	// Iterate through all bookable products to get global availability for this date
	if( $available_products ) {

		foreach( $available_products as $product_id ) {

			// Availability, e.g. This date is currently available to book
			if( ! isset( $metadata['booking_meta'][$product_id]['used_bookings'] ) ) {

				// Full availability
				$global_availability[$product_id] = __( 'Fully available', 'bfwc' );
				$used_bookings[$product_id] = 0;

			} else {

				$max = bfwc_get_max_bookings( $product_id );
				$used = $metadata['booking_meta'][$product_id]['used_bookings'];
				$used_bookings[$product_id] = $used;

				if( $used >= $max ) {

					$global_availability[$product_id] = __( 'No availability', 'bfwc' );

				} else {

					$remaining = $max - $used;
					$global_availability[$product_id] = sprintf(
						__( 'Partial availability: %d remaining', 'bfwc' ),
						$remaining
					);

				}

				// Get bookings for this date
				$booking_ids = ! empty( $metadata['booking_meta'][$product_id]['booking_ids'] ) ? $metadata['booking_meta'][$product_id]['booking_ids'] : false;

				if( $booking_ids ) {

					$bookings_by_date = bfwc_get_bookings_by_date( $date, $metadata, $product_id );
					if( $bookings_by_date ) {
						$booking_data[$product_id] = $bookings_by_date;
					}
					
				}

			}

			$blocked_by = '';
			if( ! empty( $metadata['blocked_meta'][$product_id]['blocked_by'] ) ) {

				// This date is blocked out by an external service
				// @todo

				$global_availability[$product_id] .= '&nbsp;';
				$global_availability[$product_id] .= sprintf(
					__( 'Blocked by %s', 'bfwc' ),
					$metadata['blocked_meta'][$product_id]['blocked_by']
				);

			}

		}

	}

	wp_send_json( array(
		'date' 				=> $formatted_date,
		'availability'		=> $global_availability,
		'booking_data'		=> $booking_data,
		'used'				=> $used_bookings,
		'booked_day_url'	=> $booked_day_id > 0 ? get_admin_url().'/post.php?post=' . $booked_day_id . '&action=edit' : ''
	) );

}
add_action( 'wp_ajax_bfwc_get_global_booking_data', 'bfwc_get_global_booking_data' );

/**
 * Return an array of availability for all bookable products per day
 * This is used in the global calendar
 * @since 2.0.0
 */
function bfwc_get_global_availability_per_day( $available_products=false ) {

	if( ! $available_products ) {
		$available_products = bfwc_get_all_bookable_products();
	}

	$global_availability = array();

	// Iterate through each bookable product to get global availability per date
	if( $available_products ) {

		foreach( $available_products as $product_id ) {

			$max_bookings = bfwc_get_max_bookings( $product_id );
			$start_date = new DateTime( 'NOW' );

			// Check if there's a last date
			$last_date = bfwc_get_last_bookable_date( $product_id );
			if( $last_date ) {

				// Check if the date is valid (e.g. could be non-English date string)
				if( ! strtotime( $last_date ) ) {
					// Try to get valid date
					$last_date = bfwc_get_last_bookable_date_ymd( $product_id );
				}
				$end_date = new DateTime( $last_date );

			} else {

				// Find the last booked date
				// Anything after that is fully available
				$last_booked_day = bfwc_get_last_booked_date_by_product( $product_id );
				if( $last_booked_day ) {
					$end_date = new DateTime( $last_booked_day );
				} else {
					$end_date = new DateTime( 'NOW' );
				}

			}

			$start_date_ymd = $start_date->format( 'Y-m-d' );
			$end_date_ymd = $end_date->format( 'Y-m-d' );

			// Iterate through each date from start to end
			// Use $check_date then add one day
			$check_date = new DateTime( $start_date_ymd );
			$check_date = $check_date->format( 'Y-m-d' );

			do {

				$check_date = new DateTime( $check_date );
				$date = $check_date->format( 'Y-m-d' );

				// Does this date have any used bookings?
				$used_bookings = bfwc_get_available_bookings( $date, $product_id );

				// Store the available bookings
				$global_availability[$date][$product_id] = array(
					'max'	=> $max_bookings,
					'used'	=> $used_bookings
				);

				if( $used_bookings >= $max_bookings ) {
					// Status is unavailable
					$global_availability[$date][$product_id]['status'] = 'unavailable';
				} else if( $used_bookings > 0 ) {
					// Status is part available
					$global_availability[$date][$product_id]['status'] = 'part-available';
				} else {
					$global_availability[$date][$product_id]['status'] = 'fully-available';
				}

				$check_date->modify( '+1 day' );
				$check_date = $check_date->format( 'Y-m-d' );

			} while( $check_date <= $end_date_ymd );

		}

	}

	return $global_availability;

}

/**
 * Return an array of availability statuses for all bookable products per day
 * This is used in the global calendar
 * Each day is assigned a status - available, partly available or unavailable
 * @since 2.0.0
 */
function bfwc_get_global_availability_status_per_day() {

	$available_products = bfwc_get_all_bookable_products();
	$number_products = count( $available_products );
	$global_availability = bfwc_get_global_availability_per_day( $available_products );

	$availability_statuses = array();

	// Iterate through each date with a booking
	if( $global_availability ) {

		// $availabiilty is an array by product ID showing max, used and status for the date
		foreach( $global_availability as $date=>$availability ) {

			// Use this to set the status per date
			$date_status = array();

			// We need to iterate through each product ID per date to get its status
			foreach( $availability as $product_id=>$data ) {

				// This is the status for this product on this date
				$date_status[] = $data['status'];

			}

			// This is the number of product statuses for this date
			$total_statuses = count( $date_status );

			// This is the number of each status for this date, e.g. part-available => 1, fully-available => 1, unavailable => 2
			$count_values = array_count_values( $date_status );

			// If all statuses are unavailable, the date is unavailable
			if( isset( $count_values['unavailable'] ) && $count_values['unavailable'] == $number_products ) {
				$availability_statuses[$date] = 'unavailable';
			} else if( ! isset( $count_values['unavailable'] ) && ! isset( $count_values['part-available'] ) ) {
				$availability_statuses[$date] = 'fully-available';
			} else {
				$availability_statuses[$date] = 'part-available';
			}

		}

	}

	return $availability_statuses;

}

/**
 * Get details of booking for specific date
 * Used in the global calendar
 * @since 2.0.0
 */
function bfwc_get_bookings_by_date( $date, $metadata=false, $product_id=false ) {
	
	// Step through each date in the month
	if( ! $metadata ) {
		$metadata = bfwc_get_booked_day_metadata( $date );
	}

	// Return this array
	$booking_data = array();

	$booking_ids = ! empty( $metadata['booking_meta'][$product_id]['booking_ids'] ) ? $metadata['booking_meta'][$product_id]['booking_ids'] : false;

	if( $booking_ids ) {
		// @todo duplicate code
		foreach( $booking_ids as $id ) {

			// Only published bookings
			// if( get_post_status( $id ) != 'publish' ) {
			// 	continue;
			// }

			$booking_url = add_query_arg(
				array(
					'post'			=> $id,
					'post_type'		=> 'bfwc_booking',
					'action'		=> 'edit'
				),
				admin_url( 'post.php' )
			);
			$product_url = add_query_arg(
				array(
					'post'			=> $product_id,
					'post_type'		=> 'product',
					'action'		=> 'edit'
				),
				admin_url( 'post.php' )
			);
	
			$args = array(
				'id'					=> $id,
				'booking_url'			=> $booking_url,
				'product_url'			=> $product_url,
				'start'					=> bfwc_format_date( get_post_meta( $id, 'bfwc_start_date', true ) ),
				'end'					=> bfwc_format_date( get_post_meta( $id, 'bfwc_end_date', true ) )
			);
	
			// 2.0.5
			$order_id = get_post_meta( $id, 'bfwc_order_id', true );
			if ( $order_id ) {
				$args['order_id'] = $order_id;
				$order = wc_get_order( $order_id );
				if ( $order ) {
					if ( bfwc_hpos_enabled() ) {
						$order_url = $order->get_edit_order_url(); // WC HPOS compatible
					} else {
						$order_url = add_query_arg(
							array(
								'post'			=> $order_id,
								'action'		=> 'edit'
							),
							admin_url( 'post.php' )
						);
					}
					$args['order_url'] = $order_url;
				} else {
					$args['order_url'] = '';
				}
			} else {
				$args['order_id'] = $args['order_url'] = '';
			}

			$user_id = get_post_meta( $id, 'bfwc_user_id', true );
	
			if( $user_id ) {
				$user = get_user_by( 'id', $user_id );
				if ( bfwc_hpos_enabled() ) {
					$user_url = add_query_arg(
						array(
							'page'				=> 'wc-orders',
							'_customer_user'	=> $user_id
						),
						admin_url( 'admin.php' )
					);
				} else {
					$user_url = add_query_arg(
						array(
							'post_type'			=> 'shop_order',
							'_customer_user'	=> $user_id
						),
						admin_url( 'edit.php' )
					);
				}
				$args['user_id']	= $user_id;
				$args['user_name']	= apply_filters( 'bfwc_user_nicename', $user->user_nicename );
				$args['user_url']	= $user_url;
	
			} else if ( isset( $order ) ) {
				// 2.0.5, maybe a guest user
				$args['user_name'] = trim( $order->get_billing_first_name().' '.$order->get_billing_last_name() );
				$args['user_url']  = '';
			} else {
				$args['user_name'] = '';
				$args['user_url']  = ''; 
			}
	
			$booking_data[] = $args;
	
		}

	}

	return $booking_data;

}

/**
 * Get details of booking per day in specified month
 * Used in the global calendar
 * @since 2.0.0
 */
function bfwc_get_bookings_by_month( $month, $year ) {

	// Step through each date in the month and build this array
	$booking_data = array();

	$start_date = new DateTime( '01-' . $month . '-' . $year );
	$start_date_ymd = $start_date->format( 'Y-m-d' );
	$end_date_ymd = $start_date->format( 'Y-m-t' );

	// Iterate through each date from start to end
	// Use $check_date then add one day
	$check_date = new DateTime( $start_date_ymd );
	$check_date = $check_date->format( 'Y-m-d' );

	do {

		$check_date = new DateTime( $check_date );
		$date = $check_date->format( 'Y-m-d' );
		$count_bookings = 0;

		$metadata = bfwc_get_booked_day_metadata( $date );

		if( ! empty( $metadata['blocked_meta'] ) ) {

			// This date is blocked by an external calendar
			$count_bookings++;

		}
		if( ! empty( $metadata['booking_meta'] ) ) {

			$booking_ids = array();
			foreach( $metadata['booking_meta'] as $product_id=>$data ) {
				if( ! empty( $data['booking_ids'] ) ) {
					$booking_ids = array_merge( $booking_ids, $data['booking_ids'] );
				}
			}

			$bookings_ids = array_unique( $booking_ids );
			$count_bookings += count( $bookings_ids );

			$booking_data[$date] = array(
				'booking_ids'	=> $bookings_ids, // All the IDs of bookings on this date
				'num_bookings'	=> $count_bookings, // Number of bookings on this day
			);

		}

		$check_date->modify( '+1 day' );
		$check_date = $check_date->format( 'Y-m-d' );

	} while( $check_date <= $end_date_ymd );

	return $booking_data;

}

/**
 * Just a wrapper for bfwc_get_bookings_by_month when admin changes month on global calendar
 * @since 2.0.0
 */
function bfwc_get_bookings_by_month_ajax() {

	// Check nonce and permissions @todo
	// if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_calendar_modal_nonce' ) ) {
	// 	wp_send_json( array( 'result' => 'nonce fail' ) );
	// 	exit;
	// }

	$month = $_REQUEST['month'] ? $_REQUEST['month'] : false;
	$year = $_REQUEST['year'] ? $_REQUEST['year'] : false;

	if( ! $month || ! $year ) {
		wp_send_json( array( 'result' => 'missing dates' ) );
		exit;
	}

	$booking_data = bfwc_get_bookings_by_month( $month, $year );
	
	wp_send_json( array(
		'booking_data'	=> $booking_data
	) );

}
add_action( 'wp_ajax_bfwc_get_bookings_by_month_ajax', 'bfwc_get_bookings_by_month_ajax' );

/**
 * Get maximum availability per product
 * @since 2.0.0
 */
function bfwc_get_max_bookings_per_product() {

	$max = array();
	$products = bfwc_get_all_bookable_products();

	if( $products ) {

		foreach( $products as $product_id ) {

			$max[$product_id] = bfwc_get_max_bookings( $product_id );

		}
	}

	return $max;

}