<?php
/**
 * Functions for emails
 * @since 2.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Send an email when the quote form is submitted
 */
function bfwc_email_classes( $email_classes ) {

	require( trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'classes/class-wc-before-start-email.php' );
	require( trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'classes/class-wc-after-start-email.php' );
	require( trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'classes/class-wc-before-end-email.php' );
	require( trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'classes/class-wc-after-end-email.php' );

	// add the email class to the list of email classes that WooCommerce loads
	$email_classes['WC_Before_Start_Booking_Email'] = new WC_Before_Start_Booking_Email();	
	$email_classes['WC_After_Start_Booking_Email'] = new WC_After_Start_Booking_Email();
	$email_classes['WC_Before_End_Booking_Email'] = new WC_Before_End_Booking_Email();
	$email_classes['WC_After_End_Booking_Email'] = new WC_After_End_Booking_Email();

	return $email_classes;

}
add_filter( 'woocommerce_email_classes', 'bfwc_email_classes' );

/**
 * Check for emails to send
 * @since 2.0.0
 */
function bfwc_scheduled_emails() {

	// Check for emails to send before bookings start
	$before_start_settings = get_option( 'woocommerce_wc_before_start_booking_settings', array() );
	$days_before = ! empty( $before_start_settings['days'] ) ? $before_start_settings['days'] : false;
	$is_enabled = ! empty( $before_start_settings['enabled'] ) ? $before_start_settings['enabled'] : false;
	if( $is_enabled == 'yes' && $days_before ) {

		// Checking for today + $days_before for bookings that will start then
		$date = date( 'Y-m-d', strtotime( '+' . absint( $days_before ) . 'days' ) );

		$args = array(
			'post_type'			=> 'bfwc_booking',
			'posts_per_page'	=> -1,
			'meta_query' 		=> array(
				'relation'		=> 'AND',
				array(
					'key'     => 'bfwc_start_date',
					'value'   => $date,
					'compare' => '='
				),
				array(
					'key'     => 'bfwc_status',
					'value'   => 'cancelled',
					'compare' => '!='
				)
			),
			'fields'		=> 'ids'
		);
		$bookings = new WP_Query( $args );
	
		// A list of bookings
		$booking_ids = $bookings->posts;
		if( $booking_ids ) {
			foreach( $booking_ids as $booking_id ) {
				
				// Send the email
				do_action( 'bfwc_trigger_before_booking_starts_email', $booking_id, $date );
				// Get WooCommerce email objects
	
				$mailer = WC()->mailer()->get_emails();
				// Send the email with custom heading & subject
				$mailer['WC_Before_Start_Booking_Email']->trigger( $booking_id, $date );
	
			}
		}
		wp_reset_postdata();

	}

	// Check for emails to send after bookings start
	$after_start_settings = get_option( 'woocommerce_wc_after_start_booking_settings', array() );
	$days_after = ! empty( $after_start_settings['days'] ) ? $after_start_settings['days'] : false;
	$is_enabled = ! empty( $after_start_settings['enabled'] ) ? $after_start_settings['enabled'] : false;
	if( $is_enabled == 'yes' && $days_after ) {

		// Checking for today + $days_after
		$date = date( 'Y-m-d', strtotime( '-' . absint( $days_after ) . 'days' ) );

		$args = array(
			'post_type'			=> 'bfwc_booking',
			'posts_per_page'	=> -1,
			'meta_query' 		=> array(
				'relation'		=> 'AND',
				array(
					'key'     => 'bfwc_start_date',
					'value'   => $date,
					'compare' => '='
				),
				array(
					'key'     => 'bfwc_status',
					'value'   => 'cancelled',
					'compare' => '!='
				)
			),
			'fields'		=> 'ids'
		);
		$bookings = new WP_Query( $args );
	
		// A list of bookings that started $days_after days ago
		$booking_ids = $bookings->posts;
		if( $booking_ids ) {
			foreach( $booking_ids as $booking_id ) {
	
				$mailer = WC()->mailer()->get_emails();
				$mailer['WC_After_Start_Booking_Email']->trigger( $booking_id, $date );
	
			}
		}
		wp_reset_postdata();

	}

	// Check for emails to send before bookings end
	$before_end_settings = get_option( 'woocommerce_wc_before_end_booking_settings', array() );
	$days_before = ! empty( $before_end_settings['days'] ) ? $before_end_settings['days'] : false;
	$is_enabled = ! empty( $before_end_settings['enabled'] ) ? $before_end_settings['enabled'] : false;
	if( $is_enabled == 'yes' && $days_before ) {

		// Checking for today + $days_before for bookings that will start then
		$date = date( 'Y-m-d', strtotime( '+' . absint( $days_before ) . 'days' ) );

		$args = array(
			'post_type'			=> 'bfwc_booking',
			'posts_per_page'	=> -1,
			'meta_query' 		=> array(
				'relation'		=> 'AND',
				array(
					'key'     => 'bfwc_end_date',
					'value'   => $date,
					'compare' => '='
				),
				array(
					'key'     => 'bfwc_status',
					'value'   => 'cancelled',
					'compare' => '!='
				)
			),
			'fields'		=> 'ids'
		);
		$bookings = new WP_Query( $args );
	
		// A list of bookings
		$booking_ids = $bookings->posts;
		if( $booking_ids ) {
			foreach( $booking_ids as $booking_id ) {
				
				// Send the email
				do_action( 'bfwc_trigger_before_booking_ends_email', $booking_id, $date );
				// Get WooCommerce email objects
	
				$mailer = WC()->mailer()->get_emails();
				// Send the email with custom heading & subject
				$mailer['WC_Before_End_Booking_Email']->trigger( $booking_id, $date );
	
			}
		}
		wp_reset_postdata();

	}

	// Check for emails to send after bookings end
	$after_end_settings = get_option( 'woocommerce_wc_after_end_booking_settings', array() );
	$days_after = ! empty( $after_end_settings['days'] ) ? $after_end_settings['days'] : false;
	$is_enabled = ! empty( $after_end_settings['enabled'] ) ? $after_end_settings['enabled'] : false;
	if( $is_enabled == 'yes' && $days_after ) {

		// Checking for today + $days_before for bookings that will start then
		$date = date( 'Y-m-d', strtotime( '-' . absint( $days_after ) . 'days' ) );

		$args = array(
			'post_type'			=> 'bfwc_booking',
			'posts_per_page'	=> -1,
			'meta_query' 		=> array(
				'relation'		=> 'AND',
				array(
					'key'     => 'bfwc_end_date',
					'value'   => $date,
					'compare' => '='
				),
				array(
					'key'     => 'bfwc_status',
					'value'   => 'cancelled',
					'compare' => '!='
				)
			),
			'fields'		=> 'ids'
		);
		$bookings = new WP_Query( $args );
	
		// A list of bookings
		$booking_ids = $bookings->posts;
		if( $booking_ids ) {
			foreach( $booking_ids as $booking_id ) {
				
				// Send the email
				do_action( 'bfwc_trigger_after_booking_ends_email', $booking_id, $date );
				// Get WooCommerce email objects
	
				$mailer = WC()->mailer()->get_emails();
				// Send the email with custom heading & subject
				$mailer['WC_After_End_Booking_Email']->trigger( $booking_id, $date );
	
			}
		}
		wp_reset_postdata();

	}
	

}
add_action( 'woocommerce_scheduled_sales', 'bfwc_scheduled_emails' );

function bfwc_replace_tags_in_emails( $content, $email ) {

	if( ! empty( $email->booking_id ) ) {
		$content = str_replace( '{booking_id}', $email->booking_id, $content );
	}
	if( ! empty( $email->date ) ) {
		$date = bfwc_format_date( $email->date );
		$content = str_replace( '{date}', $date, $content );
	}
	
	return $content;

}