<?php
/**
 * Functions for booking notes
 * @since 2.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Create a new note on a booking page
 * @since 2.0.0
 */
function bfwc_create_new_note( $message, $booking_id ) {

	// Notes are saved as a custom post type
	if( $message ) {

		$date_format = get_option( 'date_format' );
		$title = date( $date_format . ' H:i:s' );

		$notes_data = array(
			'post_status'    => 'publish',
			'post_type'      => 'bfwc_booking_notes',
			'post_author'    => get_current_user_id(),
			'post_title'     => $title,
			'post_content'   => sanitize_text_field( $message ),
			'comment_status' => 'closed',
		);
		$note_id = wp_insert_post( $notes_data );

		// Assign the booking ID
		update_post_meta( $note_id, 'bfwc_booking_id', $booking_id );

		return $note_id;

	}

}

/**
 * Add a new note
 * @since 2.0.0
 */
function bfwc_add_new_note() {

	// Check nonce and permissions
	if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_add_note' ) ) {
		wp_send_json( array( 'result' => 'nonce fail' ) );
		exit;
	}

	$message = $_POST['message'] ? $_POST['message'] : false;
	$booking_id = $_POST['booking_id'] ? $_POST['booking_id'] : false;

	$note_id = bfwc_create_new_note( $message, $booking_id );

	if( $note_id ) {
		wp_send_json( array(
			'note_id' 			=> $note_id,
			'date'					=> get_the_title( $note_id ),
			'message'				=> sanitize_text_field( $message )
		) );
	} else {
		wp_send_json( array( 'result' => 'note failed' ) );
	}

	exit;

}
add_action( 'wp_ajax_bfwc_add_new_note', 'bfwc_add_new_note' );
