<?php
/**
 * Functions for creating orders based on bookings
 * @since 2.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Create a new order from a booking page
 * @since 2.0.0
 */
function bfwc_create_new_order() {

	// Create a new order
	if( isset( $_GET['create_order'] ) && $_GET['create_order'] == 'true' ) {

		$booking_id = ! empty( $_GET['post'] ) ? $_GET['post'] : false;
		if( ! $booking_id ) {
			return;
		}

		$product_id = get_post_meta( $booking_id, 'bfwc_product_id', true );
		if( ! $product_id ) {
			return;
		}

		// Check if the order already exists
		$order_id = get_post_meta( $booking_id, 'bfwc_order_id', true );
		if( $order_id ) {
			return;
		}

		// Create the order
		$order = wc_create_order();
		$order_id = $order->get_id();
		//update_post_meta( $order_id, 'bfwc_booking_id', $booking_id );
		$order->update_meta_data( 'bfwc_booking_id', $booking_id ); // WC HPOS compatibility

		$cost = get_post_meta( $booking_id, 'bfwc_cost', true );

		$product = wc_get_product( $product_id );
		if ( 'incl' === get_option( 'woocommerce_tax_display_shop' ) && ! apply_filters( 'bfwc_disable_tax_adjustment_create_order', false, $product ) ) {
			// 2.0.4, remove tax from the cost
			add_filter( 'woocommerce_prices_include_tax', '__return_true' ); // this is needed if woocommerce_prices_include_tax is 'no', so that wc_get_price_excluding_tax() computes for the non-taxed value
			$cost = wc_get_price_excluding_tax( $product, array( 'qty' => 1, 'price' => $cost ) );
			remove_filter( 'woocommerce_prices_include_tax', '__return_true' );
		}

		$order->add_product( $product, 1, array( 'subtotal' => $cost, 'total' => $cost ) );

		// Assign user ID to order
		$user_id = get_post_meta( $booking_id, 'bfwc_booking_assigned_user', true );
		if( $user_id ) {
			$order->set_customer_id( $user_id );
		}

		$order->set_status( 'wc-pending' );

		// Calculate the order total
		$order->calculate_totals();

		// Assign order ID to booking
		update_post_meta( $booking_id, 'bfwc_order_id', $order_id );

		// Create payment URL to send to customer
		// @todo - option to automatically send this link to customer
		//update_post_meta( $order_id, 'bfwc_payment_link', $order->get_checkout_payment_url() );
		$order->update_meta_data( 'bfwc_payment_link', $order->get_checkout_payment_url() ); // WC HPOS

		$order->save(); // save order meta data to DB

	}

}
add_action( 'admin_init', 'bfwc_create_new_order' );

/**
 * Add wc-pending as status for payment
 * @since 2.0.0
 */
function bfwc_valid_order_statuses_for_payment( $statuses, $instance ) {

	// Add pending so that we can send payment links to customers for bookings manually created
	$status = array( 'wc-pending' );
	return array_merge( $statuses, $status );

}
add_filter( 'woocommerce_valid_order_statuses_for_payment', 'bfwc_valid_order_statuses_for_payment', 10, 2 );
