<?php
/**
 * Functions for the admin
 * @since 1.3.4
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Before WooCommerce High-Performance Order Storage (HPOS) activation (i.e. orders using the post.php page)
 */
function bfwc_filter_orders() {
	global $typenow;
	if ( 'shop_order' === $typenow ) { ?>
		<select name="bfwc_filter_orders" id="bfwc_filter_orders">
			<option value="">
				<?php esc_html_e( 'Filter by booking', 'bfwc' ); ?>
			</option>
			<option value="bookings" <?php echo esc_attr( isset( $_GET['bfwc_filter_orders'] ) ? selected( 'bookings', $_GET['bfwc_filter_orders'], false ) : '' ); ?>>
				<?php _e( 'Bookings', 'bfwc' ); ?>
			</option>
			<option value="sales" <?php echo esc_attr( isset( $_GET['bfwc_filter_orders'] ) ? selected( 'sales', $_GET['bfwc_filter_orders'], false ) : '' ); ?>>
				<?php _e( 'Sales', 'bfwc' ); ?>
			</option>
		</select>
		<?php
	}
}
add_action( 'restrict_manage_posts', 'bfwc_filter_orders' );

/**
 * When WC HPOS is active, this is used
 * @since 2.0.3
 */
function bfwc_filter_orders_hpos() {
	?>
		<select name="bfwc_filter_orders" id="bfwc_filter_orders">
			<option value="">
				<?php esc_html_e( 'Filter by booking', 'bfwc' ); ?>
			</option>
			<option value="bookings" <?php echo esc_attr( isset( $_GET['bfwc_filter_orders'] ) ? selected( 'bookings', $_GET['bfwc_filter_orders'], false ) : '' ); ?>>
				<?php _e( 'Bookings', 'bfwc' ); ?>
			</option>
			<option value="sales" <?php echo esc_attr( isset( $_GET['bfwc_filter_orders'] ) ? selected( 'sales', $_GET['bfwc_filter_orders'], false ) : '' ); ?>>
				<?php _e( 'Sales', 'bfwc' ); ?>
			</option>
		</select>
	<?php
}
add_action( 'woocommerce_order_list_table_restrict_manage_orders', 'bfwc_filter_orders_hpos' );

/**
 * Update query params for different filters
 * @since 2.0.0
 */
function bfwc_sort_orders_by_booking( $query ) {
  global $pagenow;

  // Only display orders that are a booking
  // This is used before WC HPOS activation
  $post_type = isset( $_GET['post_type'] ) ? $_GET['post_type'] : '';
  if ( is_admin() && $pagenow=='edit.php' && $post_type == 'shop_order' && isset( $_GET['bfwc_filter_orders'] ) && $_GET['bfwc_filter_orders'] == 'bookings' ) {
    $query->query_vars['meta_key'] = 'bfwc_booking_id';
    // $query->query_vars['meta_value'] = $_GET['slug'];
    $query->query_vars['meta_compare'] = 'EXISTS';
  } else if ( is_admin() && $pagenow=='edit.php' && $post_type == 'shop_order' && isset( $_GET['bfwc_filter_orders'] ) && $_GET['bfwc_filter_orders'] == 'sales' ) {
    $query->query_vars['meta_key'] = 'bfwc_booking_id';
    // $query->query_vars['meta_value'] = '';
    $query->query_vars['meta_compare'] = 'NOT EXISTS';
  }

	// Filter bookings by user
	// Not orders, so this is needed even when using HPOS
	if( 'edit.php' === $pagenow && isset( $_GET['post_type'] ) && 'bfwc_booking' === $_GET['post_type'] && ! empty( $_GET['_customer_user'] ) ) {
		$user_id = $_GET['_customer_user'];
		$query->query_vars['meta_key'] = 'bfwc_user_id';
		$query->query_vars['meta_value'] = $user_id;
		$query->query_vars['meta_compare'] = 'EXISTS';
	}

	// Filter bookings by product ID
	if( 'edit.php' === $pagenow && isset( $_GET['post_type'] ) && 'bfwc_booking' === $_GET['post_type'] && ! empty( $_GET['product_id'] ) ) {
		$product_id = $_GET['product_id'];
		$query->query_vars['meta_key'] = 'bfwc_product_id';
		$query->query_vars['meta_value'] = $product_id;
		$query->query_vars['meta_compare'] = 'EXISTS';
	}

}
add_filter( 'parse_query', 'bfwc_sort_orders_by_booking' );

/**
 * Update query params for different filters (WC HPOS compatible)
 * @since 2.0.3
 */
function bfwc_order_query_args_hpos( $args ) {

	if ( ! empty( $_GET['bfwc_filter_orders'] ) ) {
		if ( 'bookings' === $_GET['bfwc_filter_orders'] ) {
			$args['meta_query'] = array(
				array(
					'key'			=> 'bfwc_booking_id',
					'compare'	=> 'EXISTS'
				),
			);
		} else if ( 'sales' === $_GET['bfwc_filter_orders'] ) {
			$args['meta_query'] = array(
				array(
					'key'			=> 'bfwc_booking_id',
					'compare'	=> 'NOT EXISTS'
				),
			);
		}
	}

	// TODO: maybe new filter for modifying order count? 'woocommerce_' . $this->order_type . '_list_table_order_count'
	// found in count_orders_by_status() function in woocommerce/src/Internal/Admin/Orders/ListTable.php

	return $args;

}
add_filter( 'woocommerce_order_query_args', 'bfwc_order_query_args_hpos' );

/**
 * When an item is removed from an order, remove any booking details associated with it
 * @since 1.4.0
 */
function bfwc_ajax_order_items_removed( $item_id, $item, $changed_stock, $order ) {

	$product_id = $item->get_product_id();
	$product = wc_get_product( $product_id );

	if( is_object( $product ) ) {

		// If this is a booking
		if( $product->get_type() == 'simple_booking' ) {

			$order_id = $order->get_id();
			// Get the associated booking ID
			//$booking_id = get_post_meta( $order_id, 'bfwc_booking_id', true );
			$booking_id = $order->get_meta( 'bfwc_booking_id', true ); // WC HPOS compatibility

			// Change the status of the booking to cancelled / removed ?
			//$start_date = get_post_meta( $order_id, 'bfwc_start_date', true );
			$start_date = $order->get_meta( 'bfwc_start_date', true ); // WC HPOS
			//$end_date = get_post_meta( $order_id, 'bfwc_end_date_plus_buffer', true );
			$end_date = $order->get_meta( 'bfwc_end_date_plus_buffer', true ); // WC HPOS

			//if( get_post_meta( $order_id, 'bfwc_num_bookings', true ) ) {
			//	$num_bookings = get_post_meta( $order_id, 'bfwc_num_bookings', true );
			//} else {
			//	$num_bookings = 1;
			//}
			$num_bookings = $order->get_meta( 'bfwc_num_bookings', true ); // WC HPOS
			if ( empty( $num_bookings ) ) $num_bookings = 1;

			$booking_meta['start_date'] = $start_date;
			$booking_meta['end_date'] = $end_date;
			$booking_meta['num_bookings'] = $num_bookings;

			// Make dates available again
			bfwc_update_booked_days( $product_id, $booking_id, $booking_meta, 'restore' );

		}

	}

}
// add_action( 'woocommerce_ajax_order_items_removed', 'bfwc_ajax_order_items_removed', 10, 4 );

/**
 * When an order is cancelled, cancel the booking
 * @since 1.4.0
 */
function bfwc_order_status_cancelled( $order_id, $order=false ) {

	if ( ! is_a( $order, 'WC_Order' ) ) {
		$order = wc_get_order( $order_id );
	}
	// Check for a booking associated with the order
	//$booking_id = get_post_meta( $order_id, 'bfwc_booking_id', true );
	$booking_id = $order->get_meta( 'bfwc_booking_id', true ); // WC HPOS compatibility
	if( $booking_id ) {
		bfwc_restore_cancelled_dates( $order_id, $booking_id );
	}

}
add_action( 'woocommerce_order_status_cancelled', 'bfwc_order_status_cancelled' );
add_action( 'woocommerce_order_status_refunded', 'bfwc_order_status_cancelled' );

/**
 * Restore cancelled dates so that they are available again
 * @since 1.4.0
 */
function bfwc_restore_cancelled_dates( $order_id, $booking_id ) {

	$product_id = get_post_meta( $booking_id, 'bfwc_product_id', true );
	$start_date = get_post_meta( $booking_id, 'bfwc_start_date', true );
	//$end_date = get_post_meta( $booking_id, 'bfwc_end_date_plus_buffer', true ); // commented out 1.4.7
	$end_date = get_post_meta( $booking_id, 'bfwc_end_date', true ); // since 1.4.7. we pass the real end date because the buffer dates don't have bfwc_booked_days records and booking_meta to adjust

	if( get_post_meta( $booking_id, 'bfwc_num_bookings', true ) ) {
		$num_bookings = get_post_meta( $booking_id, 'bfwc_num_bookings', true );
	} else {
		$num_bookings = 1;
	}

	$booking_meta['start_date'] = $start_date;
	$booking_meta['end_date'] = $end_date;
	$booking_meta['num_bookings'] = $num_bookings;

	// Make dates available again
	bfwc_update_booked_days( $product_id, $booking_id, $booking_meta, 'restore' );

	// Change the status of the booking to cancelled
	update_post_meta( $booking_id, 'bfwc_status', 'cancelled' );

}

/**
 * Display the booking status
 * @param $echo Boolean Whether to echo the status
 * @since 1.4.0
 */
function bfwc_display_booking_status( $booking_id, $echo=true ) {
	$status = get_post_meta( $booking_id, 'bfwc_status', true );
	$status_html = '';
	if( $status == 'pending' ) {
		$status_html = '<mark class="order-status status-pending tips"><span>' . __( 'Pending', 'bfwc' ) . '</span></mark>';
	} else if( $status == 'processing' ) {
		$status_html = '<mark class="order-status status-processing tips"><span>' . __( 'Processing', 'bfwc' ) . '</span></mark>';
	} else if ( $status == 'confirmed' ) {
		$status_html = '<mark class="order-status status-completed tips"><span>' . __( 'Confirmed', 'bfwc' ) . '</span></mark>';
	} else if ( $status == 'cancelled' ) {
		$status_html = '<mark class="order-status status-cancelled tips"><span>' . __( 'Cancelled', 'bfwc' ) . '</span></mark>';
	} else if ( $status == 'removed' ) {
		$status_html = '<mark class="order-status status-cancelled tips"><span>' . __( 'Removed', 'bfwc' ) . '</span></mark>';
	} else if ( $status == 'manual' ) {
		$status_html = '<mark class="order-status status-bfwc-manual tips"><span>' . __( 'Manual', 'bfwc' ) . '</span></mark>';
	} else if ( $status == 'edited' ) {
		$status_html = '<mark class="order-status status-bfwc-edited tips"><span>' . __( 'Edited', 'bfwc' ) . '</span></mark>';
	}
	if( $echo ) {
		echo $status_html;
	} else {
		return $status_html;
	}
}

/**
 * Get the remaining availability for a specific date range
 * Used when editing a booking from the order screen
 * @since 1.4.0
 */
function bfwc_get_availability_by_range() {

	// Check nonce and permissions
	if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_edit_booking' ) ) {
		wp_send_json( array( 'result' => 'nonce fail' ) );
		exit;
	}

	$start_date = ! empty( $_POST['start_date_ymd'] ) ? $_POST['start_date_ymd'] : false;
	$end_date = ! empty( $_POST['end_date_ymd'] ) ? $_POST['end_date_ymd'] : false;
	$product_id = ! empty( $_POST['product_id'] ) ? $_POST['product_id'] : false;
	$booking_id = ! empty( $_POST['booking_id'] ) ? $_POST['booking_id'] : false;

	if( ! $start_date || ! $end_date || ! $product_id ) {
		wp_send_json( array( 'result' => 'missing data' ) );
		exit;
	}

	// This gets the remaining availability for the date range
	// It does not account for availability already assigned to this booking - need to fix that
	// Possibly need to assign booking_id=>quantity per booked_day
	
	$available = bfwc_get_maximum_booking_by_range( $product_id, $start_date, $end_date, $booking_id );

	wp_send_json( array( 'available' => $available ) );

}
add_action( 'wp_ajax_bfwc_get_availability_by_range', 'bfwc_get_availability_by_range' );

/**
 * Used when editing a booking from the order screen - deprecated 2.0
 * @since 1.4.0
 */
function bfwc_save_edited_booking() {
	wc_deprecated_function( 'bfwc_save_edited_booking', '2.0', 'bfwc_edit_booking' );
}

/**
 * When editing a booking, restore current booking dates in order to make them available for editing
 * @since 2.0.0
 */
function bfwc_edit_booking() {

	// Check nonce and permissions
	if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_save_booking' ) ) {
		wp_send_json( array( 'result' => 'nonce fail' ) );
		exit;
	}

	// $start_date = $_POST['start_date'] ? $_POST['start_date'] : false;
	// $end_date = $_POST['end_date'] ? $_POST['end_date'] : false;
	$original_start_date = ! empty( $_POST['original_start_date'] ) ? $_POST['original_start_date'] : false;
	$original_end_date = ! empty( $_POST['original_end_date'] ) ? $_POST['original_end_date'] : false;
	$booking_id = ! empty( $_POST['booking_id'] ) ? $_POST['booking_id'] : false;
	$quantity = ! empty( $_POST['quantity'] ) ? $_POST['quantity'] : 1;
	$original_quantity = ! empty( $_POST['original_quantity'] ) ? $_POST['original_quantity'] : $quantity;
	$product_id = ! empty( $_POST['product_id'] ) ? $_POST['product_id'] : false;

	if( ! $booking_id || ! $original_quantity || ! $product_id ) {
		wp_send_json( array( 'result' => 'missing data' ) );
		exit;
	}

	// Restore availability for original dates
	$booking_meta['start_date'] = $original_start_date;
	$booking_meta['end_date'] = $original_end_date;
	$booking_meta['num_bookings'] = $original_quantity;
	bfwc_update_booked_days( $product_id, $booking_id, $booking_meta, 'restore' );

	$availability = bfwc_get_availability_per_day( $product_id );

	// Set option to say this booking ID is being edited
	$currently_edited = bfwc_get_currently_edited_ids();
	if( $currently_edited ) {
		$currently_edited[] = $booking_id;
	} else {
		$currently_edited = array( $booking_id );
	}
	update_option( 'bfwc_currently_edited', $currently_edited );

	// Add note to booking

	do_action( 'bfwc_booking_edited', $booking_id );

	wp_send_json( array( 'availability' => $availability ) );

}
add_action( 'wp_ajax_bfwc_edit_booking', 'bfwc_edit_booking' );

/**
 * When editing a booking is cancelled, restore original dates
 * @since 2.0.0
 */
function bfwc_cancel_editing_booking() {

	// Check nonce and permissions
	if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'bfwc_save_booking' ) ) {
		wp_send_json( array( 'result' => 'nonce fail' ) );
		exit;
	}

	// $start_date = $_POST['start_date'] ? $_POST['start_date'] : false;
	// $end_date = $_POST['end_date'] ? $_POST['end_date'] : false;
	$original_start_date = $_POST['original_start_date'] ? $_POST['original_start_date'] : false;
	$original_end_date = $_POST['original_end_date'] ? $_POST['original_end_date'] : false;
	$booking_id = $_POST['booking_id'] ? $_POST['booking_id'] : false;
	$quantity = $_POST['quantity'] ? $_POST['quantity'] : false;
	$original_quantity = $_POST['original_quantity'] ? $_POST['original_quantity'] : $quantity;
	$product_id = $_POST['product_id'] ? $_POST['product_id'] : false;

	if( ! $booking_id || ! $original_quantity || ! $product_id ) {
		wp_send_json( array( 'result' => 'missing data' ) );
		exit;
	}

	// Update availability for new dates
	$booking_meta['start_date'] = $original_start_date;
	$booking_meta['end_date'] = $original_end_date;
	$booking_meta['num_bookings'] = $original_quantity;
	bfwc_update_booked_days( $product_id, $booking_id, $booking_meta, 'std' );

	// Delete flag to say this is being edited
	$currently_edited = bfwc_get_currently_edited_ids();
	if( ( $key = array_search( $booking_id, $currently_edited)) !== false ) {
		unset( $currently_edited[$key] );
	}
	update_option( 'bfwc_currently_edited', $currently_edited );


	// Add note to booking

	do_action( 'bfwc_after_cancel_edit', $booking_id );

	wp_send_json( array( 'done' ) );

}
add_action( 'wp_ajax_bfwc_cancel_editing_booking', 'bfwc_cancel_editing_booking' );

/**
 * Add a field to filter bookings by user
 * @since 2.0.0
 */
function bfwc_filter_bookings() {
	global $typenow;
	if ( 'bfwc_booking' === $typenow ) {

		$user_string = '';
		$user_id     = '';

		// phpcs:disable WordPress.Security.NonceVerification.Recommended
		if ( ! empty( $_GET['_customer_user'] ) ) {
			$user_id = absint( $_GET['_customer_user'] );
			$user    = get_user_by( 'id', $user_id );

			$user_string = sprintf(
				/* translators: 1: user display name 2: user ID 3: user email */
				esc_html__( '%1$s (#%2$s &ndash; %3$s)', 'bfwc' ),
				$user->display_name,
				absint( $user->ID ),
				$user->user_email
			);
		} ?>
		<select class="wc-customer-search" name="_customer_user" data-placeholder="<?php esc_attr_e( 'Filter by customer', 'bfwc' ); ?>" data-allow_clear="true">
			<option value="<?php echo esc_attr( $user_id ); ?>" selected="selected"><?php echo htmlspecialchars( wp_kses_post( $user_string ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></option>
		</select>

		<select class="wc-product-search" style="min-width: 150px;" name="product_id" data-placeholder="<?php esc_attr_e( 'Filter by product', 'bfwc' ); ?>" data-action="woocommerce_json_search_products">
			<?php
			$product_id = ! empty( $_GET['product_id'] ) ? $_GET['product_id'] : false;

			if( $product_id ) {
				$product = wc_get_product( $product_id );
				if ( is_object( $product ) ) {
					echo '<option value="' . esc_attr( $product_id ) . '"' . selected( true, true, false ) . '>' . esc_html( wp_strip_all_tags( $product->get_formatted_name() ) ) . '</option>';
				}
			}
			?>
		</select>

		<?php
	}
}
add_action( 'restrict_manage_posts', 'bfwc_filter_bookings' );

/**
 * Get a list of booking IDs that need saving
 * @since 2.0.0
 */
function bfwc_get_currently_edited_ids() {

	$currently_edited = get_option( 'bfwc_currently_edited', array() );
	$currently_edited = array_unique( $currently_edited );
	return $currently_edited;

}

/**
 * Get a list of booking IDs that need saving
 * @since 2.0.0
 */
function bfwc_get_validation_fails() {

	$validation_fails = get_option( 'bfwc_validation_fails', array() );
	if( $validation_fails ) {
		// $validation_fails = array_unique( $validation_fails );
	}
	
	return $validation_fails;

}

/**
 * Display a notice if there are any bookings that are still being edited
 * @since	2.0.0
 * @version 2.0.4
 */
function bfwc_currently_edited_notice() {
	$currently_edited = bfwc_get_currently_edited_ids();
	if( $currently_edited ) {
		// Exclude any trashed or non-existent items
		foreach( $currently_edited as $index=>$id ) {
			$post_status = get_post_status( $id );
			// $post_status === false added in 2.0.4, to remove permanently deleted bookings
			if( $post_status === false || $post_status == 'trash' ) {
				unset( $currently_edited[$index]);
			}
		}
		if( ! $currently_edited ) {
			return;
		}
		$bookings_text = array();
		foreach( $currently_edited as $id ) {
			$url = add_query_arg(
				array(
					'post'			=> $id,
					'post_type'	=> 'bfwc_booking',
					'action'		=> 'edit'
				),
				admin_url( 'post.php' )
			);
			$bookings_text[] = sprintf(
				'<li><a href="%s">#%s</a></li>',
				esc_url( $url ),
				$id
			);
		} ?>
		<div class="notice notice-error">
			<p><?php _e( 'The following bookings have been edited but not saved. Please ensure you save the bookings correctly.', 'bfwc' ); ?></p>
			<ul>
				<?php echo join( '', $bookings_text ); ?>
			</ul>
		</div>
	<?php }
}
add_action( 'admin_notices', 'bfwc_currently_edited_notice' );


/**
 * Display a notice if there are any bookings that fail validation
 * @since 2.0.0
 */
function bfwc_validation_fail_notice() {
	$validation_fails = bfwc_get_validation_fails();
	if( $validation_fails ) {
		// Exclude any trashed items
		foreach( $validation_fails as $id=>$missing_data ) {
			$post_status = get_post_status( $id );
			// $post_status === false added in 2.0.4, to remove permanently deleted bookings
			if( $post_status === false || $post_status == 'trash' ) {
				unset( $validation_fails[$id]);
			}
		}
		if( ! $validation_fails ) {
			return;
		}
		$bookings_text = array();
		foreach( $validation_fails as $id=>$missing_data ) {
			$url = add_query_arg(
				array(
					'post'			=> $id,
					'post_type'		=> 'bfwc_booking',
					'action'		=> 'edit'
				),
				admin_url( 'post.php' )
			);
			$data_string = join( ', ', $missing_data );
			$bookings_text[] = sprintf(
				'<li><a href="%s">#%s</a>: %s</li>',
				esc_url( $url ),
				$id,
				$data_string
			);
			$post = array(
				'ID' => $id,
				'post_status' => 'draft'
			);
			wp_update_post( $post );
		} ?>
		<div class="notice notice-error">
			<p><?php _e( 'The following bookings have some data missing. Please ensure you edit each one to add all required data to avoid any issues with your bookings.', 'bfwc' ); ?></p>
			<ul>
				<?php echo join( '', $bookings_text ); ?>
			</ul>
		</div>
	<?php }
}
add_action( 'admin_notices', 'bfwc_validation_fail_notice' );
