<?php
/**
 * Functions for setting up the Booking product type
 * @since 1.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Add the Simple Booking product type to dropdown list
 * @param $types	List of product types
 */
function bfwc_add_booking_type( $types ) {
	$types['simple_booking'] = __( 'Booking', 'bfwc' );
	return $types;
}
add_filter( 'product_type_selector', 'bfwc_add_booking_type' );

/**
 * Simple Booking product type parameters
 * @param $tabs	List of tabs
 */
function bfwc_product_tabs( $tabs ) {
	$tabs['booking'] = array(
		'label'		=> __( 'Booking', 'bfwc' ),
		'target'	=> 'booking_options',
		'class'		=> array( 'show_if_simple_booking', 'show_if_variable_booking' ),
		'priority'	=> 1
	);
	return $tabs;
}
add_filter( 'woocommerce_product_data_tabs', 'bfwc_product_tabs' );

/**
 * Contents of the booking options product tab.
 */
function bfwc_booking_options_tab_content() {

	global $post; ?>

	<div id='booking_options' class='panel woocommerce_options_panel'>

		<div class="bfwc_navigation">
			<ul>
				<?php printf(
					'<li data-id="general" id="bfwc_general_menu_item" class="bfwc-menu-item active"><a href="#">%s</a></li>',
					__( 'General', 'bfwc' )
				);
				$show_calendar = get_option( 'bfwc_backend_calendar', 'yes' );
				if( $show_calendar == 'yes' ) {
					printf(
						'<li data-id="availability" id="bfwc_availability_menu_item" class="bfwc-menu-item"><a href="#">%s</a></li>',
						__( 'Availability', 'bfwc' )
					);
				}
				printf(
					'<li data-id="pricing" id="bfwc_pricing_menu_item" class="bfwc-menu-item"><a href="#">%s</a></li>',
					__( 'Custom Pricing', 'bfwc' )
				);
				printf(
					'<li data-id="discounts" id="bfwc_discounts_menu_item" class="bfwc-menu-item"><a href="#">%s</a></li>',
					__( 'Discounts', 'bfwc' )
				);
				printf(
					'<li data-id="external" id="bfwc_external_menu_item" class="bfwc-menu-item"><a href="#">%s</a></li>',
					__( 'External Calendars', 'bfwc' )
				); ?>
			</ul>
		</div>

		<div class='options_group bfwc_options_group bfwc_general active'>

			<div class="options-group-inner">

				<?php printf( '<h2>%s</h2>', __( 'General', 'bfwc' ) ); ?>
				<?php

				woocommerce_wp_select( array(
					'id'		=> '_booking_unit',
					'label'		=> __( 'Booking unit', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'Either daily, overnight or weekly', 'bfwc' ),
					'options'	=> array(
						'day'		=> __( 'Day', 'bwfc' ),
						'night'		=> __( 'Night', 'bwfc' ),
						'week'		=> __( 'Week', 'bwfc' )
					)
				) );

				woocommerce_wp_text_input( array(
					'id'			=> '_max_bookings',
					'label'			=> __( 'Max bookings per unit (quantity)', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'Number of times that this product can be booked out on the same day/night/week', 'bfwc' ),
					'type' 			=> 'number',
					'custom_attributes'	=> array(
						'min'	=> 1,
					),
					'default'		=> 1
				) );

				// woocommerce_wp_checkbox( array(
				// 	'id'			=> '_book_multiple',
				// 	'label'			=> __( 'Book multiple?', 'bfwc' ),
				// 	'desc_tip'		=> 'true',
				// 	'description'	=> __( 'Allow users to book multiple instances of this product simultaneously', 'bfwc' ),
				// 	'class'				=> 'bfwc_show_if_max_bookings bfwc_checkbox_left'
				// ) );

				woocommerce_wp_checkbox( array(
					'id'					=> '_allow_overlaps',
					'label'				=> __( 'Allow overlaps', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'Only for overnight bookings - allow a booking to end the same day that another starts', 'bfwc' ),
					'class'				=> 'bfwc_show_if_overnight bfwc_checkbox_left'
				) );

				woocommerce_wp_select( array(
					'id'		=> '_bookings_start',
					'label'		=> __( 'Bookings start', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'Day of the week for a booking to start', 'bfwc' ),
					'options'	=> array(
						'99'	=> __( 'Any', 'bwfc' ),
						'0'		=> __( 'Sunday', 'bwfc' ),
						'1'		=> __( 'Monday', 'bwfc' ),
						'2'		=> __( 'Tuesday', 'bwfc' ),
						'3'		=> __( 'Wednesday', 'bwfc' ),
						'4'		=> __( 'Thursday', 'bwfc' ),
						'5'		=> __( 'Friday', 'bwfc' ),
						'6'		=> __( 'Saturday', 'bwfc' ),
						'7'		=> __( 'Sunday', 'bwfc' )
					)
				) );

				woocommerce_wp_text_input( array(
					'id'			=> '_min_length',
					'label'			=> __( 'Min booking length', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'Minimum number of days or weeks allowed per booking', 'bfwc' ),
					'type' 			=> 'number',
					'custom_attributes'	=> array(
						'min'	=> 1
					)
				) );

				woocommerce_wp_text_input( array(
					'id'			=> '_max_length',
					'label'			=> __( 'Max booking length', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'Leave blank or at 0 if there is no maximum length', 'bfwc' ),
					'type' 			=> 'number',
					'custom_attributes'	=> array(
						'min'	=> 0
					)
				) );

				woocommerce_wp_text_input( array(
					'id'			=> '_postpone_start',
					'label'			=> __( 'Postpone start', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'The number of days before a user can start the booking - works like a minimum notice period for you', 'bfwc' ),
					'type' 			=> 'number',
					'custom_attributes'	=> array(
						'min'	=> 0,
						'max'	=> 999
					)
				) );

				woocommerce_wp_text_input( array(
					'id'			=> '_buffer',
					'label'			=> __( 'Buffer period', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'Set a buffer period after each booking', 'bfwc' ),
					'type' 			=> 'number',
					'custom_attributes'	=> array(
						'min'	=> 0,
						'max'	=> 6
					)
				) );

				woocommerce_wp_text_input( array(
					'id'			=> '_std_booking_cost',
					'label'			=> __( 'Standard price', 'bfwc' ),
					'desc_tip'		=> 'true',
					'description'	=> __( 'The default price per booking unit for a booking', 'bfwc' ),
					'type' 			=> 'number',
					'custom_attributes'	=> array(
						'min'	=> 0,
						'step'	=> 0.01
					)
				) );

				// Role-based prices
				if( bfwc_get_role_price_fields() ) {

					foreach( bfwc_get_role_price_fields() as $role ) {

						woocommerce_wp_text_input( array(
							'id'			=> '_std_booking_cost_' . $role,
							'label'			=> __( 'Standard cost (' . $role . ')', 'bfwc' ),
							'desc_tip'		=> 'true',
							'description'	=> __( 'The default cost per booking unit for a booking', 'bfwc' ),
							'type' 			=> 'number',
							'custom_attributes'	=> array(
								'min'	=> 0,
								'step'	=> 0.01
							)
						) );

					}
				}

				$last_date = get_post_meta( $post->ID, '_last_date', true );
				$last_date_ymd = get_post_meta( $post->ID, '_last_date_ymd', true ); ?>

				<p class="form-field _last_date_field">
					<label for="_last_date"><?php _e( 'Last bookable date', 'bfwc' ); ?></label>
					<span class="woocommerce-help-tip" data-tip="<?php _e( 'Don\'t allow bookings beyond this date', 'bfwc' ); ?>"></span>
					<input type="text" class="bfwc-start-date bfwc-last-date" id="_last_date" name="_last_date" value="<?php echo esc_attr( $last_date ); ?>">
					<input type="hidden" id="_last_date_ymd" name="_last_date_ymd" value="<?php echo esc_attr( $last_date_ymd ); ?>">
				</p>

			</div>

		</div>

		<?php $show_calendar = get_option( 'bfwc_backend_calendar', 'yes' );
		if( $show_calendar == 'yes' ) { ?>

			<div class="options_group bfwc_options_group bfwc_availability">
				<div class="options-group-inner">

					<?php printf( '<h2>%s</h2>', __( 'Availability', 'bfwc' ) ); ?>

					<div class="bfwc-datepicker-wrapper">

						<div id="bfwc-datepicker" class="bfwc-datepicker"></div>
						<div id="bfwc-calendar-modal">
							<?php printf(
								'<p><span class="dashicons dashicons-info-outline"></span>&nbsp;%s</p>',
								__( 'Click on a date on the calendar to view additional information', 'bfwc' )
							); ?>
						</div>
						<input type="hidden" id="bfwc_selected_date" name="bfwc_selected_date">
						<?php wp_nonce_field( 'bfwc_calendar_modal_nonce', 'bfwc_calendar_modal_nonce' ); ?>

					</div>

					<table class="bfwc-availability-key">
						<tr>
							<td class="availability-key full-availability">&nbsp;<?php _e( 'Full availability', 'bfwc' ); ?></td>
							<td class="availability-key part-availability">&nbsp;<?php _e( 'Part availability', 'bfwc' ); ?></td>
							<td class="availability-key no-availability">&nbsp;<?php _e( 'No availability', 'bfwc' ); ?></td>
						</tr>
					</table>

				</div>
			</div>

		<?php } ?>

		<div class="options_group bfwc_options_group bfwc_pricing">
			<div class="options-group-inner">
				<?php printf( '<h2>%s</h2>', __( 'Custom Pricing', 'bfwc' ) ); ?>
				<?php printf( '<p class="description">%s</p>', __( 'Use this field to add date ranges where the standard cost doesn\'t apply.', 'bfwc' ) ); ?>
				<?php printf( '<p class="description">%s</p>', __( 'Set the cost to 0 to prevent dates from being booked.', 'bfwc' ) ); ?>
				<?php printf( '<p class="description">%s</p>', __( 'The higher the date in this list, the higher its priority.', 'bfwc' ) ); ?>
				<table class="widefat non-std-data">
					<thead>
						<tr>
							<th width="1%">&nbsp;</th>
							<th><?php _e( 'Start Date', 'bfwc' ); ?></th>
							<th><?php _e( 'End Date', 'bfwc' ); ?></th>
							<?php $label = bfwc_get_unit_label(); ?>
							<th><?php _e( 'Cost per ', 'bfwc' );
							echo esc_html( $label['single'] ); ?></th>
							<th class="remove" width="1%">&nbsp;</th>
						</tr>
					</thead>
					<tfoot>
						<tr>
							<th colspan="5">
								<p><a href="#" class="button add_new_row"><?php _e( 'Add Date Range', 'bfwc' ); ?></a></p>
							</th>
						</tr>
					</tfoot>
					<tbody id="data_rows">
						<tr class="new-data-row">
							<th class="sort-handle" width="1%"><span class="dashicons dashicons-sort"></span></th>
							<td>
								<input type="text" class="non-std-start-date bfwc-start-date" name="" value="">
								<input type="hidden" class="non-std-start-date-ymd" name="" value="">
							</td>
							<td>
								<input type="text" class="non-std-end-date bfwc-end-date" name="" value="">
								<input type="hidden" class="non-std-end-date-ymd" name="" value="">
							</td>
							<td>
								<input type="number" class="non-std-cost" step="0.01" min="0" name="" value="">
							</td>
							<th class="remove" width="1%"><span class="dashicons dashicons-no"></span></th>
						</tr>
						<?php
						$count = 0;
						/**
						 *	_nonstd_data
						 *	=> start_date
						 *	=> num_units
						 *	=> cost
						 */
						$ranges = get_post_meta( $post->ID, '_nonstd_data', true );
						// Tidy up the array to remove empty values and reset the keys
						// $ranges = array_values(array_filter( $ranges ) );
						if( $ranges ) {
							foreach( $ranges as $range ) { ?>
								<tr data-count="<?php echo absint( $count ); ?>">
									<th class="sort-handle" width="1%"><span class="dashicons dashicons-sort"></span></th>
									<td>
										<?php $value = '';
										if( isset( $range['start_date'] ) ) {
											$value = $range['start_date'];
										} ?>
										<input type="text" class="non-std-start-date bfwc-start-date" name="_nonstd_data[<?php echo absint( $count ); ?>][start_date]" value="<?php echo esc_attr( $value ); ?>">
										<?php $value = '';
										if( isset( $range['start_date_ymd'] ) ) {
											$value = $range['start_date_ymd'];
										} else if( isset( $range['start_date'] ) ) {
											// If the ymd value isn't set, use the start_date value
											$value = $range['start_date'];
										} ?>
										<input type="hidden" class="non-std-start-date-ymd" name="_nonstd_data[<?php echo absint( $count ); ?>][start_date_ymd]" value="<?php echo esc_attr( $value ); ?>">
									</td>
									<td>
										<?php $value = '';
										if( isset( $range['end_date'] ) ) {
											$value = $range['end_date'];
										} ?>
										<input type="text" class="non-std-end-date bfwc-end-date" name="_nonstd_data[<?php echo absint( $count ); ?>][end_date]" value="<?php echo esc_attr( $value ); ?>">
										<?php $value = '';
										if( isset( $range['end_date_ymd'] ) ) {
											$value = $range['end_date_ymd'];
										}  else if( isset( $range['end_date'] ) ) {
											// If the ymd value isn't set, use the start_date value
											$value = $range['end_date'];
										} ?>
										<input type="hidden" class="non-std-end-date-ymd" name="_nonstd_data[<?php echo absint( $count ); ?>][end_date_ymd]" value="<?php echo esc_attr( $value ); ?>">
									</td>
									<td>
										<?php $value = '';
										if( isset( $range['cost'] ) ) {
											$value = $range['cost'];
										} ?>
										<input type="number" class="non-std-cost" step="0.01" min="0" name="_nonstd_data[<?php echo absint( $count ); ?>][cost]" value="<?php echo esc_attr( $value ); ?>">
									</td>
									<th class="remove" width="1%"><span class="dashicons dashicons-no"></span></th>
								</tr>
							<?php
							$count++;
							}
						}
						wp_nonce_field( 'bfwc_nonstd_nonce', 'bfwc_nonstd_nonce' );
						?>
					</tbody>
				</table>
			</div>
		</div>

		<div class="options_group bfwc_options_group bfwc_discounts	">
			<div class="options-group-inner">
				<?php printf( '<h2>%s</h2>', __( 'Discounts', 'bfwc' ) ); ?>
				<?php printf( '<p class="description">%s</p>', __( 'You can set a discount rule here', 'bfwc' ) ); ?>

				<table class="widefat discount-data">
					<thead>
						<tr>
							<th width="1%">&nbsp;</th>
							<th class="bfwc-discount-column"><?php _e( 'Discount Rate', 'bfwc' ); ?></th>
							<th class="bfwc-discount-column"><?php _e( 'After', 'bfwc' ); ?></th>
							<th class="bfwc-discount-column"><?php _e( 'Apply From', 'bfwc' ); ?></th>
							<th class="remove" width="1%">&nbsp;</th>
						</tr>
					</thead>

					<tbody id="discount_rows">
						<?php
						$count = 0;
						/**
						 *	Only one discount is permitted at the moment, but that might change...
						 */
						$discounts = get_post_meta( $post->ID, '_discount_data', true );
						if( ! $discounts ) {
							$discounts = array(
								array( '', '', '' )
							);
						}
						// Tidy up the array to remove empty values and reset the keys
						// $ranges = array_values(array_filter( $ranges ) );
						/**
						 * $discount['rate'] 	=> the percentage discount applied to the product
						 * $discount['after'] => the number of units at which the discount kicks in, e.g. after 5 days
						 * $discount['from'] 	=> apply discount to the whole booking or just from the period when it kicks in
						 *
						 */
						if( $discounts ) {
							$max_discount_rate = bfwc_get_max_discount_rate( $post->ID ); // 2.0.5
							foreach( $discounts as $discount ) { ?>
								<tr data-count="<?php echo absint( $count ); ?>">
									<th class="sort-handle" width="1%"></th>
									<td class="bfwc-discount-column">
										<?php $value = '';
										if( isset( $discount['rate'] ) ) {
											$value = $discount['rate'];
										} ?>
										<input type="number" min=0 max="<?php echo $max_discount_rate; ?>" step=0.01 class="bfwc-discount-rate" name="_discount_data[<?php echo absint( $count ); ?>][rate]" value="<?php echo esc_attr( $value ); ?>">
									</td>
									<td class="bfwc-discount-column">
										<?php $value = '';
										if( isset( $discount['after'] ) ) {
											$value = $discount['after'];
										} ?>
										<input type="number" min=0 step=1 class="bfwc-discount-after" name="_discount_data[<?php echo absint( $count ); ?>][after]" value="<?php echo esc_attr( $value ); ?>">
									</td>
									<td class="bfwc-discount-column">
										<?php $value = '';
										if( isset( $discount['from'] ) ) {
											$value = $discount['from'];
										} ?>
										<select class="bfwc-discount-from" name="_discount_data[<?php echo absint( $count ); ?>][from]">
											<option value="all" <?php selected( 'all', $value, true ); ?>><?php _e( 'All', 'bfwc' ); ?></option>
											<option value="partial" <?php selected( 'partial', $value, true ); ?>><?php _e( 'Partial', 'bfwc' ); ?></option>
										</select>
									</td>
									<th class="remove" width="1%"></th>
								</tr>
							<?php
							$count++;
							}
						}
						?>
					</tbody>
				</table>


			</div>
		</div>

		<div class="options_group bfwc_options_group bfwc_external">
			<div class="options-group-inner">

				<?php printf( '<h2>%s</h2>', __( 'External Calendars', 'bfwc' ) ); ?>

				<?php
				$max_bookings = bfwc_get_max_bookings( $post->ID );
				if( $max_bookings > 1 ) {
					// Just let the user know that some features might not work when there are more than 1 slot available for this product
					printf(
						'<div class="bfwc-notice"><p>%s</p></div>',
						__( 'This product has more than 1 max booking set. This may create some issues with external services.', 'bfwc' )
					);
				}
				$unit = bfwc_get_booking_unit( $post->ID );
				if( $unit != 'day' ) {
					// Just let the user know that some features might not work when unit is not day
					printf(
						'<div class="bfwc-notice"><p>%s</p></div>',
						__( 'The Booking unit is not set to day. This may create some issues with external services.', 'bfwc' )
					);
				} ?>

				<?php
					woocommerce_wp_checkbox( array(
						'id'			=> 'bfwc_sync_calendars',
						'label'			=> __( 'Enable Sync Calendars', 'bfwc' ),
						'desc_tip'		=> 'true',
						'description'	=> __( 'Generate a calendar containing bookings for this product', 'bfwc' )
					) );
				?>

				<?php
					if( ! empty( $_GET['post'] ) ) {
						$url = esc_url( add_query_arg( array(
							'sync_cal'	=> 'true',
							'product_id'	=> $post->ID
						) ) );
						printf(
							'<p><a class="button" href="%s">%s</a></p>',
							$url,
							__( 'Sync calendars', 'bfwc' )
						);
					}

				?>

				<?php
					printf(
						'<p class="form-field _last_date_field"><label>%s</label><span class="woocommerce-help-tip" data-tip="%s"></span><code>%s</code></p>',
						__( 'Export URL', 'bfwc' ),
						__( 'The link to the calendar for this product - add this link to external services like Airbnb', 'bfwc' ),
						bfwc_get_calendar_url( $post->ID )
					);
				?>

				<table class="widefat sync-table">
					<thead>
						<tr>
							<th class="sync-table-column"><?php _e( 'Service', 'bfwc' ); ?></th>
							<th class="sync-table-column"><?php _e( 'Calendar URL', 'bfwc' ); ?></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<?php woocommerce_wp_text_input( array(
									'id'			=> 'bfwc_calendar_name_1',
									'label'			=> __( 'Name 1', 'bfwc' ),
									'desc_tip'		=> 'true',
									'description'	=> __( 'Enter the name of the service this connects to, e.g. Airbnb', 'bfwc' ),
								) ); ?>
							</td>
							<td>
								<?php woocommerce_wp_text_input( array(
									'id'			=> 'bfwc_calendar_url_1',
									'label'			=> __( 'URL 1', 'bfwc' ),
									'desc_tip'		=> 'true',
									'description'	=> __( 'Enter the .ics link here', 'bfwc' ),
								) ); ?>
							</td>
						</tr>
						<tr>
							<td>
								<?php woocommerce_wp_text_input( array(
									'id'			=> 'bfwc_calendar_name_2',
									'label'			=> __( 'Name 2', 'bfwc' ),
									'desc_tip'		=> 'true',
									'description'	=> __( 'Enter the name of the service this connects to, e.g. Airbnb', 'bfwc' ),
								) ); ?>
							</td>
							<td>
								<?php woocommerce_wp_text_input( array(
									'id'			=> 'bfwc_calendar_url_2',
									'label'			=> __( 'URL 2', 'bfwc' ),
									'desc_tip'		=> 'true',
									'description'	=> __( 'Enter the .ics link here', 'bfwc' ),
								) ); ?>
							</td>
						</tr>
						<tr>
							<td>
								<?php woocommerce_wp_text_input( array(
									'id'			=> 'bfwc_calendar_name_3',
									'label'			=> __( 'Name 3', 'bfwc' ),
									'desc_tip'		=> 'true',
									'description'	=> __( 'Enter the name of the service this connects to, e.g. Airbnb', 'bfwc' ),
								) ); ?>
							</td>
							<td>
								<?php woocommerce_wp_text_input( array(
									'id'			=> 'bfwc_calendar_url_3',
									'label'			=> __( 'URL 3', 'bfwc' ),
									'desc_tip'		=> 'true',
									'description'	=> __( 'Enter the .ics link here', 'bfwc' ),
								) ); ?>
							</td>
						</tr>
						<tr>
							<td>
								<?php woocommerce_wp_text_input( array(
									'id'			=> 'bfwc_calendar_name_4',
									'label'			=> __( 'Name 4', 'bfwc' ),
									'desc_tip'		=> 'true',
									'description'	=> __( 'Enter the name of the service this connects to, e.g. Airbnb', 'bfwc' ),
								) ); ?>
							</td>
							<td>
								<?php woocommerce_wp_text_input( array(
									'id'			=> 'bfwc_calendar_url_4',
									'label'			=> __( 'URL 4', 'bfwc' ),
									'desc_tip'		=> 'true',
									'description'	=> __( 'Enter the .ics link here', 'bfwc' ),
								) ); ?>
							</td>
						</tr>
					</tbody>
				</table>

			</div>
		</div>

	</div>
<?php }
add_action( 'woocommerce_product_data_panels', 'bfwc_booking_options_tab_content' );

/**
 * Save the custom fields.
 */
function bfwc_save_booking_option_field( $post_id ) {

	// Check nonce
	if ( isset( $_POST['_booking_unit'] ) ) :
		update_post_meta( $post_id, '_booking_unit', sanitize_text_field( $_POST['_booking_unit'] ) );
	endif;

	if ( isset( $_POST['_allow_overlaps'] ) ) {
		update_post_meta( $post_id, '_allow_overlaps', sanitize_text_field( $_POST['_allow_overlaps'] ) );
	} else {
		delete_post_meta( $post_id, '_allow_overlaps' );
	}

	if ( isset( $_POST['_book_multiple'] ) ) {
		update_post_meta( $post_id, '_book_multiple', sanitize_text_field( $_POST['_book_multiple'] ) );
	} else {
		delete_post_meta( $post_id, '_book_multiple' );
	}

	if ( isset( $_POST['_bookings_start'] ) ) :
		update_post_meta( $post_id, '_bookings_start', sanitize_text_field( $_POST['_bookings_start'] ) );
	endif;

	if ( isset( $_POST['_min_length'] ) ) :
		update_post_meta( $post_id, '_min_length', sanitize_text_field( $_POST['_min_length'] ) );
	endif;

	if ( isset( $_POST['_max_length'] ) ) :
		update_post_meta( $post_id, '_max_length', sanitize_text_field( $_POST['_max_length'] ) );
	endif;

	if ( isset( $_POST['_postpone_start'] ) ) :
		update_post_meta( $post_id, '_postpone_start', sanitize_text_field( $_POST['_postpone_start'] ) );
	endif;

	if ( isset( $_POST['_buffer'] ) ) :
		update_post_meta( $post_id, '_buffer', sanitize_text_field( $_POST['_buffer'] ) );
	endif;

	if ( isset( $_POST['_std_booking_cost'] ) ) :
		update_post_meta( $post_id, '_std_booking_cost', sanitize_text_field( $_POST['_std_booking_cost'] ) );
	endif;

	// Role-based prices
	if( bfwc_get_role_price_fields() ) {

		foreach( bfwc_get_role_price_fields() as $role ) {

			if ( isset( $_POST['_std_booking_cost_' . $role] ) ) :
				update_post_meta( $post_id, '_std_booking_cost_' . $role, sanitize_text_field( $_POST['_std_booking_cost_' . $role] ) );
			endif;

		}

	}

	if ( isset( $_POST['_last_date'] ) ) :
		update_post_meta( $post_id, '_last_date', $_POST['_last_date'] );
	endif;

	if ( isset( $_POST['_last_date_ymd'] ) ) {
		if ( empty( $_POST['_last_date'] ) ) {
			update_post_meta( $post_id, '_last_date_ymd', '' ); // 2.0.5
		} else {
			update_post_meta( $post_id, '_last_date_ymd', $_POST['_last_date_ymd'] );
		}
	}

	$max_bookings = isset( $_POST['_max_bookings'] ) ? $_POST['_max_bookings'] : 1;
	if ( isset( $_POST['_max_bookings'] ) ) :
		update_post_meta( $post_id, '_max_bookings', sanitize_text_field( $max_bookings ) );
	endif;

	if ( isset( $_POST['_nonstd_data'] ) ) {

		$nonstd_data = $_POST['_nonstd_data'];
		update_post_meta( $post_id, '_nonstd_data', $_POST['_nonstd_data'] );
		// We need to zero out the available bookings (quantity) for any dates that are unavailable
		$gather_dates = array();
		foreach( $nonstd_data as $key=>$nonstd_item ) {
			if( isset( $nonstd_item['start_date'] ) && isset( $nonstd_item['end_date'] ) && isset( $nonstd_item['cost'] ) && $nonstd_item['cost'] == 0 ) {

				$booking_meta['start_date'] = $nonstd_item['start_date_ymd'];
				$booking_meta['end_date'] = $nonstd_item['end_date_ymd'];
				$booking_meta['num_bookings'] = $max_bookings;
				bfwc_update_booked_days( $post_id, 0, $booking_meta );

			}
		}

	} else {
		// 2.0.4, check if this product has existing _nonstd_data
		$nonstd_data = get_post_meta( $post_id, '_nonstd_data', true );
		if ( $nonstd_data ) {
			// maybe restore booked day first just to be on the safe side?
			foreach( $nonstd_data as $key=>$nonstd_item ) {
				if( isset( $nonstd_item['start_date'] ) && isset( $nonstd_item['end_date'] ) && isset( $nonstd_item['cost'] ) && $nonstd_item['cost'] == 0 ) {
					$booking_meta['start_date'] = $nonstd_item['start_date_ymd'];
					$booking_meta['end_date'] = $nonstd_item['end_date_ymd'];
					$booking_meta['num_bookings'] = $max_bookings;
					bfwc_update_booked_days( $post_id, 0, $booking_meta, 'restore' );
				}
			}
			// delete
			delete_post_meta( $post_id, '_nonstd_data' );
		}
	}

	if ( isset( $_POST['_discount_data'] ) ) :
		update_post_meta( $post_id, '_discount_data', $_POST['_discount_data'] );
	endif;

	if ( isset( $_POST['bfwc_sync_calendars'] ) ) {
		update_post_meta( $post_id, 'bfwc_sync_calendars', sanitize_text_field( $_POST['bfwc_sync_calendars'] ) );
		// Create the calendar
		bfwc_create_calendar( $post_id );
	} else {
		delete_post_meta( $post_id, 'bfwc_sync_calendars' );
	}

	for( $i = 1; $i <= 2; $i++ ) {

		if ( isset( $_POST['bfwc_calendar_name_' . $i] ) ) {
			update_post_meta( $post_id, 'bfwc_calendar_name_' . $i, sanitize_text_field( $_POST['bfwc_calendar_name_' . $i] ) );
		} else {
			delete_post_meta( $post_id, 'bfwc_calendar_name_' . $i );
		}

		if ( isset( $_POST['bfwc_calendar_url_' . $i] ) ) {
			update_post_meta( $post_id, 'bfwc_calendar_url_' . $i, sanitize_url( $_POST['bfwc_calendar_url_' . $i] ) );
		} else {
			delete_post_meta( $post_id, 'bfwc_calendar_url_' . $i );
		}

	}




}
add_action( 'woocommerce_process_product_meta_simple_booking', 'bfwc_save_booking_option_field'  );
add_action( 'woocommerce_process_product_meta_variable_booking', 'bfwc_save_booking_option_field'  );

/**
 * Hide the Shipping data panel.
 */
function bwfc_hide_shipping_data_panel( $tabs) {
	$tabs['shipping']['class'][] = 'hide_if_simple_booking hide_if_variable_booking';
	return $tabs;
}
add_filter( 'woocommerce_product_data_tabs', 'bwfc_hide_shipping_data_panel' );
