<?php
/**
 * The template for the edit booking field in orders
 * @since 1.4.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

global $pagenow; ?>

<div class="bfwc-metafield bfwc-booking-details-order <?php echo $field['class']; ?>">

  <table class="widefat booking-details-table">
    <thead>
      <th colspan=2><?php _e( 'Product', 'bfwc' ); ?></th>
      <th><?php _e( 'Start', 'bfwc' ); ?></th>
			<th><?php _e( 'End (inc buffer)', 'bfwc' ); ?></th>
      <th><?php _e( 'Cost', 'bfwc' ); ?></th>
      <th><?php _e( 'Quantity', 'bfwc' ); ?></th>
			<th><?php _e( 'Status', 'bfwc' ); ?></th>

    </thead>
    <tbody>
			<td class="thumb">
				<?php echo '<div class="wc-order-item-thumbnail">' . wp_kses_post( $thumbnail ) . '</div>'; ?>
			</td>
			<td class="name">
				<?php
				if( is_object( $product ) ) {
					echo $product_link ? '<a href="' . esc_url( $product_link ) . '" class="wc-order-item-name">' . wp_kses_post( $product->get_title() ) . '</a>' : '<div class="wc-order-item-name">' . wp_kses_post( $product->get_title() ) . '</div>';
					if ( $product && $product->get_sku() ) {
						echo '<div class="wc-order-item-sku"><strong>' . esc_html__( 'SKU:', 'woocommerce' ) . '</strong> ' . esc_html( $product->get_sku() ) . '</div>';
					}
				} else if( $pagenow == 'post-new.php' ) {

					// Add a new booking manually
					// Only include booking type products
					$bookable_products = bfwc_get_all_bookable_products(); ?>
					<select class="wc-product-search" style="min-width: 50px;" name="bfwc_product_id_select" id="bfwc_product_id_select" data-placeholder="<?php esc_attr_e( 'Find product', 'bfwc' ); ?>" data-include="<?php echo wc_esc_json( wp_json_encode( $bookable_products ) ); ?>" data-action="woocommerce_json_search_products">
						<?php
						$product_id = ! empty( $_GET['product_id'] ) ? $_GET['product_id'] : false;

						if( $product_id ) {
							$product = wc_get_product( $product_id );
							if ( is_object( $product ) ) {
								echo '<option value="' . esc_attr( $product_id ) . '"' . selected( true, true, false ) . '>' . esc_html( wp_strip_all_tags( $product->get_formatted_name() ) ) . '</option>';
							}
						} ?>
					</select>

					<?php
					wp_nonce_field( 'bfwc_add_product_new_booking', 'bfwc_add_product_new_booking' );
				}
			 ?>
			</td>
      <td>
        <div class="view">
          <?php $start_date = get_post_meta( $booking_id, 'bfwc_start_date_text', true );
          	printf( '<span id="bfwc-start-date-td">%s</span>', $start_date );
			if( ! $start_date ) {
				$start_date = '';
				$start_date_ymd = '';
			} else {
				// 2.0.4, maybe bfwc_start_date already exists, use that first, so that we can avoid 
				$start_date_ymd = get_post_meta( $booking_id, 'bfwc_start_date', true );
				if ( ! $start_date_ymd ) {
					$start_date_ymd = bfwc_get_ymd( $start_date );
				}
			} ?>
        </div>
        <div class="edit" style="display: none">
    			<input type="text" autocomplete="off" name="bfwc_start_date_text_edit" id="bfwc_start_date_text_edit" class="bfwc-edit-datepicker" value="<?php echo $start_date; ?>" />
				<input type="hidden" name="start_date_ymd" id="start_date_ymd" value="">
				<input type="hidden" id="original_start_date_ymd" value="<?php echo $start_date_ymd; ?>">
    		</div>
      </td>
      <td>
        <div class="view">
          <?php $end_date = get_post_meta( $booking_id, 'bfwc_end_date_text', true );
			printf( '<span id="bfwc-end-date-td">%s</span>', $end_date );
			if( ! $end_date ) {
				$end_date = '';
				$end_date_ymd = '';
			} else {
				// 2.0.4, maybe bfwc_end_date already exists, use that first
				$end_date_ymd = get_post_meta( $booking_id, 'bfwc_end_date', true );
				if ( ! $end_date_ymd ) {
					$end_date_ymd = bfwc_get_ymd( $end_date );
				}
			} ?>
        </div>
        <div class="edit edit-end-date" style="display: none">
			<input type="text" autocomplete="off" name="bfwc_end_date_text_edit" id="bfwc_end_date_text_edit" class="bfwc-edit-datepicker" value="<?php echo $end_date; ?>" disabled />
			<input type="hidden" name="end_date_ymd" id="end_date_ymd" value="">
			<input type="hidden" id="original_end_date_ymd" value="<?php echo $end_date_ymd; ?>">
        </div>
      </td>
      <td>
        <div class="view">
          <?php $price = get_post_meta( $booking_id, 'bfwc_cost', true );
          $price = wc_price( $price );
          printf( '%s', $price ); ?>
        </div>
      </td>
        <td>
          <div class="view">
            <?php $num = get_post_meta( $booking_id, 'bfwc_num_bookings', true );
						// If this is a new booking, we don't know the product ID yet
						$max = bfwc_get_max_bookings( $product_id );
            printf( '<span id="bfwc-num-bookings-td">%s</span>', $num ); ?>
          </div>
          <div class="edit edit-max-bookings" style="display: none">
						<?php
							printf(
								'<input type="number" autocomplete="off" name="bfwc_num_bookings_edit" id="bfwc_num_bookings_edit" class="" min="1" value="%s" data-max="%s" disabled />',
								$num,
								$max
							);
						?>

						<input type="hidden" id="bfwc_original_num_bookings" value="<?php echo $num; ?>" />
          </div>
        </td>
			<td>
        <?php bfwc_display_booking_status( $booking_id ); ?>
      </td>
    </tbody>
  </table>
  <?php $status = get_post_meta( $booking_id, 'bfwc_status', true );
  if( $status != 'cancelled' && $status != 'confirmed' ) {

		if( $pagenow != 'post-new.php' ) {

			echo '<p class="bfwc-edit-booking-wrapper">';
			printf(
			'<a href="#" class="button button-primary bfwc-edit-booking">%s</a>',
			__( 'Edit booking', 'bfwc' )
			);
				// printf(
			//   '<a href="#" class="button button-primary bfwc-save-booking edit disabled" style="display: none">%s</a>',
			//   __( 'Save changes ', 'bfwc' )
			// );
				printf(
			'<a href="#" class="button button-secondary bfwc-cancel-update edit" style="display: none">%s</a>',
			__( 'Cancel', 'bfwc' )
			);
			echo '</p>';

		}

		wp_nonce_field( 'bfwc_edit_booking', 'bfwc_edit_booking' );
		wp_nonce_field( 'bfwc_save_booking', 'bfwc_save_booking' );

		printf(
			'<input type="hidden" id="bfwc_product_id" name="bfwc_product_id" value="%s">',
			$product_id
		);
		printf(
			'<input type="hidden" id="bfwc_booking_id" name="bfwc_booking_id" value="%s">',
			$booking_id
		);

		printf(
			'<input type="hidden" id="_max_bookings" name="_max_bookings" value="%s">',
			$max
		);
  } ?>
</div>

<div>

</div>
<div>
	<?php	 ?>
</div>
<div>

</div>
