<?php
/**
 * The template for the edit booking field in orders
 * @since 1.4.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
} ?>

<div class="booking_data_column">
	<h3>
		<?php esc_html_e( 'User details', 'woocommerce' ); ?>
	</h3>
	<div class="address">
		<?php

		if( ! $order ) {

			// Add option to select user
			echo 'Overwrite with order user';

			$user_string = '';
			$user_id = get_post_meta( $booking_id, 'bfwc_booking_assigned_user', true );

			// phpcs:disable WordPress.Security.NonceVerification.Recommended
			if( $user_id ) {
				$user_id = absint( $user_id );
				$user = get_user_by( 'id', $user_id );

				$user_string = sprintf(
					/* translators: 1: user display name 2: user ID 3: user email */
					esc_html__( '%1$s (#%2$s &ndash; %3$s)', 'bfwc' ),
					$user->display_name,
					absint( $user->ID ),
					$user->user_email
				);
			} ?>
			<select class="wc-customer-search" name="bfwc_booking_assigned_user" id="bfwc_booking_assigned_user" data-placeholder="<?php esc_attr_e( 'Filter by customer', 'bfwc' ); ?>" data-allow_clear="true">
				<option value="<?php echo esc_attr( $user_id ); ?>" selected="selected"><?php echo htmlspecialchars( wp_kses_post( $user_string ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></option>
			</select>

		<?php } else {

			// Display values.
			if ( $order->get_formatted_billing_address() ) {
				echo '<p>' . wp_kses( $order->get_formatted_billing_address(), array( 'br' => array() ) ) . '</p>';
			} else {
				echo '<p class="none_set"><strong>' . esc_html__( 'Address:', 'woocommerce' ) . '</strong> ' . esc_html__( 'No billing address set.', 'woocommerce' ) . '</p>';
			}

			foreach ( $order->get_address( 'billing' ) as $key => $field ) {

				$field_name = 'billing_' . $key;

				if ( isset( $field['value'] ) ) {
					$field_value = $field['value'];
				} elseif ( is_callable( array( $order, 'get_' . $field_name ) ) ) {
					$field_value = $order->{"get_$field_name"}( 'edit' );
				} else {
					$field_value = $order->get_meta( '_' . $field_name );
				}

				$field_label = '';
				if ( 'billing_phone' === $field_name ) {
					$field_label = __( 'Phone', 'bfwc' );
					$field_value = wc_make_phone_clickable( $field_value );
				} elseif ( 'billing_email' === $field_name ) {
					$field_label = __( 'Email', 'bfwc' );
					$field_value = '<a href="' . esc_url( 'mailto:' . $field_value ) . '">' . $field_value . '</a>';
				}

				if ( $field_value && $field_label ) {
					echo '<p><strong>' . esc_html( $field_label ) . ':</strong> ' . wp_kses_post( $field_value ) . '</p>';
				}

			}

			$user_id = get_post_meta( $booking_id, 'bfwc_user_id', true );

			if( $user_id ) {

				$user = get_user_by( 'id', $user_id );
				$user_url = add_query_arg(
					array(
						'user_id'	=> $user_id
					),
					admin_url( 'user-edit.php' )
				);
				printf(
					'<p><a class="button secondary-button" href="%s">%s</a></p>',
					esc_url( $user_url ),
					__( 'View user profile', 'bfwc' )
				);

				if ( bfwc_hpos_enabled() ) {
					$user_orders_url = add_query_arg(
						array(
							'page'				=> 'wc-orders',
							'_customer_user'	=> $user_id
						),
						admin_url( 'admin.php' )
					);
				} else {
					$user_orders_url = add_query_arg(
						array(
							'post_type'			=> 'shop_order',
							'_customer_user'	=> $user_id
						),
						admin_url( 'edit.php' )
					);
				}
				printf(
					'<p><a class="button secondary-button" href="%s">%s</a></p>',
					esc_url( $user_orders_url ),
					__( 'View user orders', 'bfwc' )
				);

				$bookings_url = add_query_arg(
					array(
						'post_type'			=> 'bfwc_booking',
						'_customer_user'	=> $user_id
					),
					admin_url( 'edit.php' )
				);
				printf(
					'<p><a class="button secondary-button" href="%s">%s</a></p>',
					esc_url( $bookings_url ),
					__( 'View user bookings', 'bfwc' )
				);

			}

		}

		?>

	</div>
</div>
