<?php
/**
 * Class to create our Bookings post type
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

if( ! class_exists( 'BFWC_Booking_Post_Type' ) ) {

	class BFWC_Booking_Post_Type {

		public function __construct() {
		}

		public function init() {

			require_once BFWC_DIRNAME . '/inc/metaboxes.php';
			add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ), 99 );
			add_action( 'init', array( $this, 'register_post_type' ) );
			add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ) );
			add_action( 'save_post', array( $this, 'save_booking' ), 10, 3 );

			add_action( 'wp_trash_post', array( $this, 'trash_booking' ) );
			add_action( 'untrash_post', array( $this, 'untrash_booking' ), 10, 2 );

			add_filter( 'manage_bfwc_booking_posts_columns', array( $this, 'bfwc_booking_columns' ) );
			add_action( 'manage_bfwc_booking_posts_custom_column', array( $this, 'custom_bfwc_booking_columns' ), 10, 2 );

		}

		public function enqueue_scripts() {

			global $post, $pagenow;
			if( isset( $post->ID ) ) {
				$post_id = $post->ID;
			} else {
				$post_id = false;
				// return;
			}

			$style_deps = array();
			$script_deps = array(
				'jquery',
				'jquery-ui-datepicker',
				'jquery-blockui',
				'jquery-ui-sortable'
			);
			// If this is an order or bookings screen, we need to use the product ID related to the booking
			if( 'post.php' === $pagenow && isset( $_GET['post'] ) && 'bfwc_booking' === get_post_type( $_GET['post'] ) ) {
				$booking_id = $post_id;
				$post_id = get_post_meta( $booking_id, 'bfwc_product_id', true );
				$style_deps[] = 'woocommerce_admin_styles';
				$script_deps[] = 'wc-enhanced-select';
				$script_deps[] = 'selectWoo';
			}

			if( ( 'edit.php' === $pagenow || 'post-new.php' === $pagenow ) && isset( $_GET['post_type'] ) && 'bfwc_booking' === $_GET['post_type'] ) {
				$style_deps[] = 'woocommerce_admin_styles';
				$script_deps[] = 'wc-enhanced-select';
				$script_deps[] = 'selectWoo';
			}

			$date_format = get_option( 'date_format' );
			$date_format = bfwc_get_date_format_for_js( $date_format );

			wp_enqueue_style( 'bfwc-admin-style', trailingslashit( BFWC_PLUGIN_URL ) . 'assets/css/admin-style.css', $style_deps, BFWC_PLUGIN_VERSION );
			wp_register_script( 'bfwc-admin-script', trailingslashit( BFWC_PLUGIN_URL ) . 'assets/js/admin-bookings.js', $script_deps, BFWC_PLUGIN_VERSION, true );
			wp_localize_script(
				'bfwc-admin-script',
				'bfwc_params',
				array(
					'date_format' 					=> $date_format,
					'allow_overlaps'				=> bfwc_get_allow_overlaps( $post_id ),
					'booking_unit'					=> bfwc_get_booking_unit( $post_id ),
					'bookings_unavailable'			=> bfwc_get_unavailable_dates( $post_id ),
					'blocked'						=> bfwc_get_blocked_dates_by_id( $post_id ),
					'availability_text'				=> __( 'available', 'bfwc' ),
					'bookings_start'				=> get_post_meta( $post_id, '_bookings_start', true ),
					'max_bookings'					=> get_post_meta( $post_id, '_max_bookings', true ),
					'edit_text'						=> apply_filters( 'bfwc_edit_text', __( 'Edit', 'bfwc' ) ),
					'availability_text'				=> apply_filters( 'bfwc_availability_text', __( 'Availability', 'bfwc' ) ),
					'available_text'				=> apply_filters( 'bfwc_available_text', __( 'Available', 'bfwc' ) ),
					'no_availability_text'			=> apply_filters( 'bfwc_no_availability_text', __( 'No availability data', 'bfwc' ) ),
					'bookings_text'					=> apply_filters( 'bfwc_bookings_text', __( 'Bookings', 'bfwc' ) ),
					'no_bookings_text'				=> apply_filters( 'bfwc_no_bookings_text', __( 'No bookings', 'bfwc' ) ),
					'product_text'					=> apply_filters( 'bfwc_product_text', __( 'Product', 'bfwc' ) ),
					'prices'						=> bfwc_get_prices_per_day(),
					'std_cost'						=> bfwc_get_cost( $post_id ),
					'product_id'					=> $post_id,
					'product_url_base'				=> add_query_arg(
														array(
															'post_type'		=> 'product',
															'action'		=> 'edit'
														),
														admin_url( 'post.php' )
													),
					'enable_all_dates_non_standard_costs'	=> apply_filters( 'bfwc_enable_all_dates_non_standard_costs', false, $post_id ) ? 'yes' : 'no'
				)
			);

			wp_add_inline_script(
				'bfwc-admin-script',
				'var bfwc_availability = ' . json_encode( bfwc_get_availability_per_day( $post_id ) ),
				'before'
			);
			wp_add_inline_script(
				'bfwc-admin-script',
				'var bfwc_monthly_data = ' . json_encode( bfwc_get_bookings_by_month( date( 'm' ), date( 'Y' ) ) ),
				'before'
			);

			if( isset( $_GET['page'] ) && $_GET['page'] == 'bfwc_calendar_page' ) {
				// Global calendar page
				wp_add_inline_script(
					'bfwc-admin-script',
					'var bfwc_global_availability_status = ' . json_encode( bfwc_get_global_availability_status_per_day() ),
					'before'
				);
				wp_add_inline_script(
					'bfwc-admin-script',
					'var bfwc_bookable_products = ' . json_encode( bfwc_get_bookable_products_titles() ),
					'before'
				);
				
				wp_add_inline_script(
					'bfwc-admin-script',
					'var bfwc_max_bookings = ' . json_encode( bfwc_get_max_bookings_per_product() ),
					'before'
				);
			}

			wp_enqueue_script( 'bfwc-admin-script' );

		}

		public function register_post_type() {
			$labels = array(
				'name'               => __( 'Bookings', 'bfwc' ),
				'singular_name'      => __( 'Booking', 'bfwc' ),
				'add_new'            => __( 'Add Booking', 'bfwc' ),
				'add_new_item'       => __( 'Add New Booking', 'bfwc' ),
				'edit'               => __( 'Edit', 'bfwc' ),
				'edit_item'          => __( 'Edit Booking', 'bfwc' ),
				'new_item'           => __( 'New Booking', 'bfwc' ),
				'view'               => __( 'View Booking', 'bfwc' ),
				'view_item'          => __( 'View Booking', 'bfwc' ),
				'search_items'       => __( 'Search Bookings', 'bfwc' ),
				'not_found'          => __( 'No Bookings found', 'bfwc' ),
				'not_found_in_trash' => __( 'No Bookings found in trash', 'bfwc' ),
				'parent'             => __( 'Parent Bookings', 'bfwc' ),
				'menu_name'          => _x( 'Bookings', 'Admin menu name', 'bfwc' ),
				'all_items'          => __( 'Bookings', 'bfwc' ),
			);
			$args = array(
				'labels'                    => $labels,
				'label'                     => __( 'Booking', 'bfwc' ),
				'menu_icon'					=> 'dashicons-calendar-alt',
				'public'                    => false,
				'show_ui'                   => true,
				'capability_type'           => 'product',
				'map_meta_cap'              => true,
				'publicly_queryable'        => false,
				'exclude_from_search'       => true,
				'show_in_menu'              => true,
				'menu_position'				=> apply_filters( 'bfwc_menu_position', 58 ),
				'hierarchical'              => false,
				'show_in_nav_menus'         => false,
				'rewrite'                   => false,
				'query_var'                 => false,
				'supports'                  => array( '' ),
				'has_archive'               => false,
			);
			register_post_type( 'bfwc_booking', $args );

			$labels = array(
				'name'               => __( 'Booked days', 'bfwc' ),
				'singular_name'      => __( 'Booked day', 'bfwc' ),
				'add_new'            => __( 'Add Booked day', 'bfwc' ),
				'add_new_item'       => __( 'Add New Booked day', 'bfwc' ),
				'edit'               => __( 'Edit', 'bfwc' ),
				'edit_item'          => __( 'Edit Booked day', 'bfwc' ),
				'new_item'           => __( 'New Booked day', 'bfwc' ),
				'view'               => __( 'View Booked day', 'bfwc' ),
				'view_item'          => __( 'View Booked day', 'bfwc' ),
				'search_items'       => __( 'Search Booked days', 'bfwc' ),
				'not_found'          => __( 'No Booked days found', 'bfwc' ),
				'not_found_in_trash' => __( 'No Booked days found in trash', 'bfwc' ),
				'parent'             => __( 'Parent Booked days', 'bfwc' ),
				'menu_name'          => _x( 'Booked Days', 'Admin menu name', 'bfwc' ),
				'all_items'          => __( 'Booked Days', 'bfwc' ),
			);
			$args = array(
				'labels'                     => $labels,
				'label'                      => __( 'Booked Days', 'bfwc' ),
				'public'                     => bfwc_view_booked_days(),
				'show_ui'                    => bfwc_view_booked_days(),
				'capability_type'            => 'product',
				'map_meta_cap'               => true,
				'publicly_queryable'         => false,
				'exclude_from_search'        => true,
				'show_in_menu'               => 'edit.php?post_type=bfwc_booking',
				'hierarchical'               => false,
				'show_in_nav_menus'          => false,
				'rewrite'                    => false,
				'query_var'                  => false,
				'supports'                   => array( 'title' ),
				'has_archive'                => false,
			);
			register_post_type( 'bfwc_booked_days', $args );

			// Notes post type
			$labels = array(
				'name'               => __( 'Notes', 'bfwc' ),
				'singular_name'      => __( 'Note', 'bfwc' ),
				'add_new'            => __( 'Add Note', 'bfwc' ),
				'add_new_item'       => __( 'Add New Note', 'bfwc' ),
				'edit'               => __( 'Edit', 'bfwc' ),
				'edit_item'          => __( 'Edit Note', 'bfwc' ),
				'new_item'           => __( 'New Note', 'bfwc' ),
				'view'               => __( 'View Note', 'bfwc' ),
				'view_item'          => __( 'View Note', 'bfwc' ),
				'search_items'       => __( 'Search Note', 'bfwc' ),
				'not_found'          => __( 'No Notes found', 'bfwc' ),
				'not_found_in_trash' => __( 'No Notes found in trash', 'bfwc' ),
				'parent'             => __( 'Parent Notes', 'bfwc' ),
				'menu_name'          => _x( 'Notes', 'Admin menu name', 'bfwc' ),
				'all_items'          => __( 'Notes', 'bfwc' ),
			);
			$args = array(
				'labels'                     => $labels,
				'label'                      => __( 'Notes', 'bfwc' ),
				'public'                     => false,
				'show_ui'                    => false,
				'capability_type'            => 'product',
				'map_meta_cap'               => true,
				'publicly_queryable'         => false,
				'exclude_from_search'        => true,
				'show_in_menu'               => 'woocommerce',
				'hierarchical'               => false,
				'show_in_nav_menus'          => false,
				'rewrite'                    => false,
				'query_var'                  => false,
				'supports'                   => array( 'title', 'content' ),
				'has_archive'                => false,
			);
			register_post_type( 'bfwc_booking_notes', $args );
		}

		public function bfwc_booking_columns( $columns ) {
			$title = $columns['title'];
			$date = $columns['date'];
			unset( $columns['title'] );
			unset( $columns['date'] );
			$columns['booking_status'] = __( 'Status', 'bfwc' );
			$columns['title'] = $title;
			$columns['date'] = $date;
			return $columns;
		}

		public function custom_bfwc_booking_columns( $column, $post_id ) {
			if( $column == 'booking_status' ) {
				bfwc_display_booking_status( $post_id );
			}
		}

		/**
		 * Register the metabox
		 * @since 1.0.0
		 */
		public function add_meta_box() {

			$metaboxes = bfwc_metaboxes();

			foreach( $metaboxes as $metabox ) {

				$title = $metabox['title'] . '&nbsp;&#x23;' . get_the_ID();

				add_meta_box(
					$metabox['ID'],
					$title,
					array( $this, $metabox['callback'] ),
					$metabox['screens'],
					$metabox['context'],
					$metabox['priority'],
					$metabox['fields']
				);

			}

		}

		/**
		 * Metabox callback
		 * @since 1.0.0
		*/
		public function meta_box_callback( $post, $fields ) {

			wp_nonce_field( 'save_metabox_data', 'bfwc_metabox_nonce' );

			if( $fields['args'] ) {

				foreach( $fields['args'] as $field ) {

					switch( $field['type'] ) {

						case 'text':
							$this->metabox_text_output( $post, $field );
							break;
						case 'readonly':
							$this->metabox_readonly_output( $post, $field );
							break;
						case 'textarea':
							$this->metabox_textarea_output( $post, $field );
							break;
						case 'select':
							$this->metabox_select_output( $post, $field );
							break;
						case 'checkbox':
							$this->metabox_checkbox_output( $post, $field );
							break;
						case 'divider':
							$this->metabox_divider_output( $post, $field );
							break;
						case 'description':
							$this->metabox_description_output( $post, $field );
							break;
						case 'bfwc_booking_details_callback':
							$this->bfwc_booking_details_callback( $post, $field );
							break;
						case 'product':
							$this->product_callback( $post, $field );
							break;
						case 'booking':
							$this->booking_callback( $post, $field );
							break;
						case 'bfwc_notes_callback':
							$this->bfwc_notes_callback( $post, $field );
							break;
						case 'bfwc_booked_meta_callback':
							$this->bfwc_booked_meta_callback( $post, $field );
							break;

					}

				}

			}

		}

		/**
		 * Metabox callback for text type
		 * @since 1.0.0
		 */
		public function metabox_text_output( $post, $field ) {
			$value = get_post_meta( $post->ID, $field['ID'], true ); ?>
			<div class="bfwc-metafield <?php echo $field['class']; ?>">
				<label for="<?php echo esc_attr( $field['name'] ); ?>"><?php echo esc_html( $field['title'] ); ?></label>
				<input class="widefat" type="text" id="<?php echo esc_attr( $field['name'] ); ?>" name="<?php echo esc_attr( $field['name'] ); ?>" value="<?php echo esc_attr( $value ); ?>" >
			</div>
			<?php
		}

		/**
		 * Metabox callback for readonly text type
		 * @since 1.1.4
		 */
		public function metabox_readonly_output( $post, $field ) {
			$value = get_post_meta( $post->ID, $field['ID'], true ); ?>
			<div class="bfwc-metafield <?php echo $field['class']; ?>">
				<label for="<?php echo esc_attr( $field['name'] ); ?>"><?php echo esc_html( $field['title'] ); ?></label>
				<input readonly class="widefat" type="text" id="<?php echo esc_attr( $field['name'] ); ?>" name="<?php echo esc_attr( $field['name'] ); ?>" value="<?php echo esc_attr( $value ); ?>" >
			</div>
			<?php
		}

		/**
		 * Metabox callback for textarea type
		 * @since 1.0.0
		 */
		public function metabox_textarea_output( $post, $field ) {
			$value = get_post_meta( $post->ID, $field['ID'], true ); ?>
			<div class="bfwc-metafield <?php echo $field['class']; ?>">
				<label for="<?php echo esc_attr( $field['name'] ); ?>"><?php echo esc_html( $field['title'] ); ?></label>
				<textarea class="widefat" id="<?php echo esc_attr( $field['name'] ); ?>" name="<?php echo esc_attr( $field['name'] ); ?>"><?php echo esc_html( $value ); ?></textarea>
			</div>
			<?php
		}

		/**
		 * Metabox callback for select
		 */
		public function metabox_select_output( $post, $field ) {
			$field_value = get_post_meta( $post->ID, $field['ID'], true );
			// If there's no saved value and a default value exists, set the value to the default
			// This is to ensure certain settings are set automatically
			if( empty( $field_value ) && ! empty( $field['default'] ) ) {
				$field_value = $field['default'];
			} ?>
			<div class="bfwc-metafield <?php echo $field['class']; ?>">
				<label for="<?php echo esc_attr( $field['name'] ); ?>"><?php echo esc_html( $field['title'] ); ?></label>
				<select id="<?php echo esc_attr( $field['name'] ); ?>" name="<?php echo esc_attr( $field['name'] ); ?>">
					<?php if( $field['options'] ) {
						foreach( $field['options'] as $key => $value ) { ?>
							<option value="<?php echo esc_attr( $key ); ?>" <?php selected( $field_value, $key ); ?>><?php echo esc_attr( $value ); ?></option>
						<?php }
					} ?>
				</select>
			</div>
			<?php
		}

		/**
		 * Metabox callback for booking details on order page
		 * Just link to booking itself
		 */
		public function bfwc_booking_details_callback( $post, $field ) {
			//global $post;
			if ( $post instanceof WC_Order ) {
				// WC HPOS compatibility
				$order = wc_get_order( $post->get_id() );
				$order_id = $order->get_id();
				$booking_id = $order->get_meta( 'bfwc_booking_id', true );
			} else {
				$order_id = $post->ID;
				$booking_id = get_post_meta( $post->ID, 'bfwc_booking_id', true );
			}

			?>
			<div class="bfwc-metafield <?php echo $field['class']; ?>">
				<?php
			if( $booking_id ) {
				// 2.0.6, we output this immediately so that we can change the value of the variable later. In the future maybe we can make this a comma-separated value of booking IDs
				?>
				<input type="hidden" id="bfwc_booking_id" value="<?php echo $booking_id; ?>" />
				<?php
				$booking_id_array = array( $booking_id ); // use array for easy print out?
				// 2.0.6, check if this order has multiple booking products
				// we use a separate filter in case customer deactivated bfwc_max_bookings_in_cart filter
				if ( apply_filters( 'bfwc_multiple_bookings_per_order', false ) ) {

					global $wpdb;
					$query = $wpdb->prepare( "SELECT `post_id` FROM `" . $wpdb->prefix . "postmeta` WHERE `meta_key` = 'bfwc_order_id' AND `meta_value` = %s ", $order_id );
					$results = $wpdb->get_results( $query );

					if ( ! empty( $results ) ) {
						$booking_id_array = array();
						foreach ( $results as $resbook ) {
							$booking_id_array[] = $resbook->post_id;
						}
					}

				}

				foreach ( $booking_id_array as $booking_id ) {
					$url = admin_url( 'post.php?post=' . $booking_id . '&action=edit' );
					// 2.0.5
					$date_range = '';
					$start_date = get_post_meta( $booking_id, 'bfwc_start_date', true );
					$end_date = get_post_meta( $booking_id, 'bfwc_end_date_plus_buffer', true );
					if ( $start_date && $end_date ) {
						$date_range = '<br>' . __( 'Start', 'bfwc' ).': '.bfwc_format_booking_title_date( $start_date ) . '<br>' . __( 'End (inc buffer)', 'bfwc' ) . ': ' . bfwc_format_booking_title_date( $end_date ) ;
					}

					printf(
						'<p><a href="%s">%s</a> %s</p>',
						$url,
						apply_filters( 'bfwc_booking_reference', '#' . $booking_id, $booking_id ),
						$date_range
					); 
				}

			} ?>
			</div>
		<?php
		}

		/**
		 * Metabox callback for booked meta
		 */
		public function bfwc_booked_meta_callback( $post, $field ) {
			global $post;
			$value = get_post_meta( $post->ID, 'booking_meta', true ); ?>

			<div class="bfwc-metafield <?php echo $field['class']; ?>">
				<label for="<?php echo esc_attr( $field['name'] ); ?>"><?php echo esc_html( $field['title'] ); ?></label>
				<textarea class="widefat" style="height: 500px;" id="<?php echo esc_attr( $field['name'] ); ?>" name="<?php echo esc_attr( $field['name'] ); ?>"><?php print_r( $value ); ?></textarea>

				<p>
					<?php 
					$url = add_query_arg(
						array(
							'post'			=> $post->ID,
							'action'		=> 'edit',
							'clean_date'	=> 'true',
							'_wpnonce'		=> wp_create_nonce( 'bfwc_clean_date' )
						),
						admin_url( 'post.php' )
					);
					printf(
						'<a href="%s" class="button bfwc-clean-date">%s</a>',
						$url,
						__( 'Clean this date', 'bfwc' )
					); ?>
				</p>

			</div>

		<?php }

		/**
		 * Metabox for the booking details
		 * @since 2.0.0
		 */
		public function booking_callback( $post, $field ) {

			$product_id = get_post_meta( $post->ID, $field['ID'], true );

			$booking_id = $post->ID;
			$date_format = get_option( 'date_format' );

			$product      = wc_get_product( $product_id );
			$product_link = $product ? admin_url( 'post.php?post=' . $product->get_id() . '&action=edit' ) : '';
			$thumbnail    = $product ? $product->get_image( 'thumbnail', array( 'title' => '' ), true ) : '';

			$order_id = get_post_meta( $booking_id, 'bfwc_order_id', true );
			$order = wc_get_order( $order_id );

			include trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'admin/templates/booking-details.php';

			echo '<div class="bookng-columns-wrapper">';

			include trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'admin/templates/booking-user-details.php';
			include trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'admin/templates/booking-order-details.php';

			echo '</div>';

		}

		public function product_callback( $post, $field ) {

			$product_id = get_post_meta( $post->ID, $field['ID'], true );
			$booking_id = $post->ID;
			$start_date = get_post_meta( $booking_id, 'bfwc_start_date', true );
			$end_date = get_post_meta( $booking_id, 'bfwc_end_date', true ); ?>

			<div class="woocommerce_order_items_wrapper wc-order-items-editable">
				<table cellpadding="0" cellspacing="0" class="woocommerce_order_items">
					<thead>
						<tr>
							<th></th>
							<th><?php _e( 'Product', 'bfwc' ); ?></th>
							<th><?php _e( 'Start', 'bfwc' ); ?></th>
							<th><?php _e( 'End', 'bfwc' ); ?></th>

						</tr>
					</thead>
					<tbody id="order_line_items">
						<?php
						$product      = wc_get_product( $product_id );
						$product_link = $product ? admin_url( 'post.php?post=' . $product->get_id() . '&action=edit' ) : '';
						$thumbnail    = $product ? $product->get_image( 'thumbnail', array( 'title' => '' ), false ) : '';
						?>
						<tr class="item">
							<td class="thumb">
								<?php echo '<div class="wc-order-item-thumbnail">' . wp_kses_post( $thumbnail ) . '</div>'; ?>
							</td>
							<td class="name">
								<?php
								echo $product_link ? '<a href="' . esc_url( $product_link ) . '" class="wc-order-item-name">' . wp_kses_post( $product->get_title() ) . '</a>' : '<div class="wc-order-item-name">' . wp_kses_post( $product->get_title() ) . '</div>';

								if ( $product && $product->get_sku() ) {
									echo '<div class="wc-order-item-sku"><strong>' . esc_html__( 'SKU:', 'woocommerce' ) . '</strong> ' . esc_html( $product->get_sku() ) . '</div>';
								} ?>

							</td>

							<td class="start_date">
								<?php echo '<div class="">' . $start_date . '</div>'; ?>
							</td>
							<td class="end_date">
								<?php echo '<div class="">' . $end_date . '</div>'; ?>
							</td>

						</tr>
					</tbody>

				</table>
			</div>

			<?php
		}

		/**
		 * Metabox callback for the notes
		 * @since 2.0.0
		 */
		public function bfwc_notes_callback( $post, $field ) {
			$value = get_post_meta( $post->ID, $field['ID'], true ); ?>
			<div class="bfwc-metafield <?php echo $field['class']; ?>">
				<label for="<?php echo esc_attr( $field['name'] ); ?>"><?php echo esc_html( $field['title'] ); ?></label>
				<textarea class="widefat" id="bfwc-note-message" name="<?php echo esc_attr( $field['name'] ); ?>"><?php echo esc_html( $value ); ?></textarea>
				<a href="#" id="bfwc-add-note" class="button">
					<?php _e( 'Add note', 'bfwc' ); ?>
				</a>
				<?php wp_nonce_field( 'bfwc_add_note', 'bfwc_add_note' ); ?>
			</div>
			<?php

			// Print previous notes
			$args = array(
				'post_type'				=> 'bfwc_booking_notes',
				'post_status'			=> 'publish',
				'posts_per_page'	=> -1,
		    'orderby'         => 'date',
		    'order'           => 'DESC'
			);

			// Just get booking meta for this product ID
			$args['meta_query'] = array(
				'relation'		=> 'AND',
				array(
					'key'			=> 'bfwc_booking_id',
					'value'		=> $post->ID,
					'compare'	=> 'LIKE'
				)
			);
			$notes = new WP_Query( $args );
			if( $notes->have_posts() ) {

				echo '<div id="woocommerce-order-notes">';
				echo '<ul class="order_notes booking-notes">';

				while( $notes->have_posts() ) {

					$notes->the_post();
					printf(
						'<li rel="%s" class="note system-note"><div class="note_content">%s</div><p class="meta">%s</p></li>',
						get_the_ID(),
						get_the_content(),
						get_the_title()
					);
				}

				echo '</ul>';
				echo '</div>';
			}
			wp_reset_query();
		}

		/**
		 * Save the data
		 * @since 1.0.0
		 */
		public function save_booking( $post_id, $post, $update ) {

			// If we're saving a product then reset the list of bookable products that's used on the search form
			if( get_post_type( $post_id ) == 'product' ) {

				delete_transient( 'bfwc_bookable_products' );
				$bookable_products = bfwc_get_all_bookable_products();

			}

			// Check the nonce is set
			if( ! isset( $_POST['bfwc_metabox_nonce'] ) ) {
				return;
			}

			// Verify the nonce
			if( ! wp_verify_nonce( $_POST['bfwc_metabox_nonce'], 'save_metabox_data' ) ) {
				return;
			}

			// If this is an autosave, our form has not been submitted, so we don't want to do anything.
			if( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
				return;
			}

			// Check the user's permissions.
			if( ! current_user_can( 'edit_post', $post_id ) ) {
				return;
			}

			if( get_post_type( $post_id ) == 'bfwc_booking' ) {

				if( did_action( 'save_post' ) > 1 ) {
					// Prevent infinite loop from wp_update_post
					return;
				}

				// Validation - check all our fields are set otherwise we can't proceed
				$missing_fields = array();

				$status = get_post_meta( $post_id, 'bfwc_status', true ) ? get_post_meta( $post_id, 'bfwc_status', true ) : 'manual';

				if ( 'cancelled' === $status || 'confirmed' === $status ) {
					// check if this booking was saved erroneously using BFWC < 2.0.4
					$validation_fails = bfwc_get_validation_fails();
					if ( ! isset( $validation_fails[$post_id] ) ) {
						// ensure that the booked days are still blocked
						bfwc_remove_booked_days_by_booking_id( $post_id );
						// we don't need to proceed, because it will be missing some data and can cause errors
						return;
					}
				}

				// Save the booking
				$product_id = ! empty( $_POST['bfwc_product_id'] ) ? $_POST['bfwc_product_id'] : false;
				if( $product_id ) {
					update_post_meta( $post_id, 'bfwc_product_id', absint( $product_id ) );
				} else {
					// check if maybe this is an existing booking?
					$product_id_orig = get_post_meta( $post_id, 'bfwc_product_id', true );
					if ( ! $product_id_orig ) {
						// it really is missing
						$missing_fields[] = __( 'Product', 'bfwc' );
					} else {
						// use the orig
						$product_id = $product_id_orig;
					}
				}

				$start_date = ! empty( $_POST['bfwc_start_date_text_edit'] ) ? $_POST['bfwc_start_date_text_edit'] : false;
				if( $start_date ) {
					update_post_meta( $post_id, 'bfwc_start_date_text', $start_date );
				} else {
					$missing_fields[] = __( 'Start date', 'bfwc' );
				}

				$end_date = ! empty( $_POST['bfwc_end_date_text_edit'] ) ? $_POST['bfwc_end_date_text_edit'] : false;
				if( $end_date ) {
					update_post_meta( $post_id, 'bfwc_end_date_text', $end_date );
				} else {
					// check if maybe this is an existing booking?
					$end_date_orig = get_post_meta( $post_id, 'bfwc_end_date_text', true );
					if ( ! $end_date_orig ) {
						// it really is missing
						$missing_fields[] = __( 'End date', 'bfwc' );
					} else {
						// use the orig
						$end_date = $end_date_orig;
					}
				}

				if( ! empty( $_POST['start_date_ymd'] ) ) {
					update_post_meta( $post_id, 'bfwc_start_date', $_POST['start_date_ymd'] );
					$start_date_ymd = $_POST['start_date_ymd'];
				} else if( ! empty( $_POST['original_start_date_ymd'] ) ) {
					update_post_meta( $post_id, 'bfwc_start_date', $_POST['original_start_date_ymd'] );
					$start_date_ymd = $_POST['original_start_date_ymd'];
				}

				if( ! empty( $_POST['end_date_ymd'] ) ) {
					update_post_meta( $post_id, 'bfwc_end_date', $_POST['end_date_ymd'] );
					update_post_meta( $post_id, 'bfwc_end_date_plus_buffer', $_POST['end_date_ymd'] );
					$end_date_ymd = $_POST['end_date_ymd'];
				} else if( ! empty( $_POST['original_end_date_ymd'] ) ) {
					update_post_meta( $post_id, 'bfwc_end_date', $_POST['original_end_date_ymd'] );
					update_post_meta( $post_id, 'bfwc_end_date_plus_buffer', $_POST['original_end_date_ymd'] );
					$end_date_ymd = $_POST['original_end_date_ymd'];
				}

				$num = ! empty( $_POST['bfwc_num_bookings_edit'] ) ? $_POST['bfwc_num_bookings_edit'] : false;
				if( $num ) {
					update_post_meta( $post_id, 'bfwc_num_bookings', $num );
				} else {
					// check if maybe this is an existing booking?
					$num_orig = get_post_meta( $post_id, 'bfwc_num_bookings', true );
					if ( ! $num_orig ) {
						if ( 'cancelled' === $status || 'confirmed' === $status ) {
							// admin can't update quantity anymore so let's set a default
							$num = 1;
							update_post_meta( $post_id, 'bfwc_num_bookings', $num );
						} else {
							// it really is missing
							$missing_fields[] = __( 'Quantity', 'bfwc' );
						}
					} else {
						$num = $num_orig;
					}
				}

				$user = ! empty( $_POST['bfwc_booking_assigned_user'] ) ? $_POST['bfwc_booking_assigned_user'] : false;
				if( $user ) {
					update_post_meta( $post_id, 'bfwc_booking_assigned_user', $user );
				}

				if( $status ) {
					update_post_meta( $post_id, 'bfwc_status', $status );
				}

				if ( ! empty( $missing_fields ) ) {
					// 2.0.4, don't continue with the function if there are missing fields
					// If we need to display a validation message
					$validation_fails = bfwc_get_validation_fails();
					$validation_fails[$post_id] = $missing_fields;
					update_option( 'bfwc_validation_fails', $validation_fails );

					return;
				} else {
					// Ensure that the dates are available and block them out accordingly
					$booking_meta = array(
						'start_date'	=> $start_date,
						'end_date'		=> $end_date,
						'num_bookings'	=> ! empty( $num ) ? $num : 1
					);
					bfwc_update_booked_days( $product_id, $post_id, $booking_meta, 'std', array() );
				}

				// Update booking post title with dates
				$title_str = bfwc_create_booking_title( $post_id, $product_id, $start_date, $end_date);
				$update = array(
					'ID'			=> $post_id,
					'post_title'	=> $title_str
				);
				update_post_meta( $post_id, 'bfwc_booking_title', $title_str );
				wp_update_post( $update );

				// Delete flag to say this is being edited
				$currently_edited = bfwc_get_currently_edited_ids();
				if( ( $key = array_search( $post_id, $currently_edited ) ) !== false ) {
					unset( $currently_edited[$key] );
				}
				update_option( 'bfwc_currently_edited', $currently_edited );

				// If we need to display a validation message
				$validation_fails = bfwc_get_validation_fails();
				if( $missing_fields ) {
					// this might not be needed since 2.0.4
					$validation_fails[$post_id] = $missing_fields;
					update_option( 'bfwc_validation_fails', $validation_fails );
				} else if( isset( $validation_fails[$post_id] ) ) {
					// Esnure this booking is removed from the list
					unset( $validation_fails[$post_id] );
					update_option( 'bfwc_validation_fails', $validation_fails );
				}

				// Set the cost of the booking
				$start_date_ymd = bfwc_get_ymd( $start_date );
				$end_date_ymd = bfwc_get_ymd( $end_date );

				$params = array(
					'start_date' => $start_date,
					'end_date' => $end_date,
					'product_id' => $product_id,
					'pending_dates' => array(),
					'start_date_ymd' => $start_date_ymd,
					'end_date_ymd' => $end_date_ymd,
					'num_bookings' => $num,
					'calculated_booking_cost' => false,
					'num_units_int' => false
				);

				$return_values = bfwc_calculate_booking_cost( $params );
				$cost = $return_values['cost'];
				update_post_meta( $post_id, 'bfwc_cost', $cost );

				$added_note = bfwc_create_new_note( 'Saved new booking', $post_id );

				// Sync the calendar if the product has sync enabled
				do_action( 'bfwc_after_create_new_booking', $post_id, $status, $booking_meta, $product_id );

			}

		}

		/**
		 * Delete a booking - so clear the dates
		 * @since 2.0.0
		 */
		public function trash_booking( $post_id='' ) {

			if( did_action( 'wp_trash_post' ) > 1 ) {
				// Prevent bookings restoring dates multiple times
				return;
			}

	    if ( isset( $_GET['post'] ) && is_array( $_GET['post'] ) ) {

			// We're bulk trashing bookings
			foreach ( $_GET['post'] as $post_id ) {

				// Restore the dates
				if( get_post_type( $post_id ) != 'bfwc_booking' ) {
					continue;
				}

				bfwc_restore_booked_days_by_booking_id( $post_id );

        	}

	    } else {

				 // Delete single booking
				 if( get_post_type( $post_id ) != 'bfwc_booking' ) {
					 return;
				 }

				 bfwc_restore_booked_days_by_booking_id( $post_id );

	    }

			// Check for undo trash @todo

		}

		public function untrash_booking( $post_id, $previous_status ) {

			if( did_action( 'untrash_post' ) > 1 ) {
				// Prevent bookings restoring dates multiple times
				return;
			}

			bfwc_remove_booked_days_by_booking_id( $post_id );

		}

	}

	$BFWC_Booking_Post_Type = new BFWC_Booking_Post_Type;
	$BFWC_Booking_Post_Type->init();

}
