<?php
/**
 * Class to create Bookings tab in Settings
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

if( ! class_exists( 'BFWC_Booking_Settings_Tab' ) ) {

	class BFWC_Booking_Settings_Tab {

		public function __construct() {
		}

		public function init() {
			add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_tab' ), 50 );
			add_action( 'woocommerce_settings_tabs_bookings', array( $this, 'settings_tab' ) );
			add_action( 'woocommerce_sections_bookings', array( $this, 'output_sections' ) );
			add_action( 'woocommerce_update_options_bookings', array( $this, 'update_settings' ) );

			add_action( 'woocommerce_admin_field_bfwc_role_price_fields', array( $this, 'role_price_fields' ) );
			add_filter( 'woocommerce_admin_settings_sanitize_option_bfwc_role_price_fields', array( $this, 'sanitize_role_price_fields' ), 10, 3 );

			add_action( 'woocommerce_admin_field_bfwc_license_key', array( $this, 'license_key' ) );
		}

		public function add_settings_tab( $settings_tabs ) {
			$settings_tabs['bookings'] = __( 'Bookings', 'bfwc' );
			return $settings_tabs;
		}

		public function settings_tab() {
			woocommerce_admin_fields( $this->get_settings() );
		}

		public function update_settings() {
			woocommerce_update_options( $this->get_settings() );

			/**
			 * Check for auto syncing external calendars
			 * @since 2.0.0
			 */
			bfwc_auto_sync_external_calendars();
		}

		public function get_settings() {

			global $current_section;
			$settings = array();

			if( $current_section == 'bfwc' || ! $current_section ) {

				$settings = array(
					'section_title' => array(
						'name'     => __( 'Bookings for WooCommerce', 'bfwc' ),
						'type'     => 'title',
						'desc'     => '',
						'id'       => 'bfwc_settings_title'
					),
					'bfwc_block_after_order' => array(
						'name'			=> __( 'Block dates after order placed', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'		=> true,
						'desc'			=> __( 'By default, dates are blocked out when the user adds a booking to their cart. Select this option to delay blocking dates until after the user has placed the order.', 'bfwc' ),
						'id'			=> 'bfwc_block_after_order',
						'default'		=> 'no',
						'std'			=> 'no'
					),
					'bfwc_cart_expiry' => array(
						'name'			=> __( 'Cart expiry (mins)', 'bfwc' ),
						'type'			=> 'number',
						'desc_tip'		=> true,
						'desc'			=> __( 'If the option above is not selected, define how many minutes before a booking is deleted from the cart', 'bfwc' ),
						'id'			=> 'bfwc_cart_expiry',
						'default'		=> 30,
						'custom_attributes'	=> array(
							'min'		=> 10,
							'step'		=> 1
						)
					),
					'bfwc_role_price_fields' => array(
						'type'			=> 'bfwc_role_price_fields',
						'id'				=> 'bfwc_role_price_fields'
					),
					'section_end' => array(
						'type' => 'sectionend',
						'id' => 'bfwc_settings_title'
					),
				);

			} else if( $current_section == 'bfwc_calendar' ) {

				$settings = array(
					'section_title_calendar' => array(
						'name'     => __( 'Calendar settings', 'bfwc' ),
						'type'     => 'title',
						'desc'     => '',
						'id'       => 'bfwc_settings_title_calendar'
					),
					'bfwc_optimise_colours' => array(
						'name'			=> __( 'Optimise calendar colours', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'	=> true,
						'desc'			=> __( 'Use green for available dates; red for unavailable dates.', 'bfwc' ),
						'id'				=> 'bfwc_optimise_colours',
						'default'		=> 'no',
						'std'				=> 'no'
					),
					'bfwc_show_availability' => array(
						'name'			=> __( 'Show availability per day', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'	=> true,
						'desc'			=> __( 'Enable this if you allow multiple bookings per unit.', 'bfwc' ),
						'id'				=> 'bfwc_show_availability',
						'default'		=> 'no',
						'std'				=> 'no'
					),
					'bfwc_show_prices' => array(
						'name'			=> __( 'Show prices per day', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'	=> true,
						'desc'			=> __( 'Enable this to show the daily or overnight price per date.', 'bfwc' ),
						'id'				=> 'bfwc_show_prices',
						'default'		=> 'no',
						'std'				=> 'no'
					),
					'bfwc_backend_calendar' => array(
						'name'			=> __( 'Show backend calendar', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'	=> true,
						'desc'			=> __( 'Enable this to show a calendar in each product admin.', 'bfwc' ),
						'id'				=> 'bfwc_backend_calendar',
						'default'		=> 'yes',
						'std'				=> 'yes'
					),
					'bfwc_first_day' => array(
						'name'			=> __( 'Start week on', 'bfwc' ),
						'type'			=> 'select',
						'desc_tip'	=> true,
						'desc'			=> __( 'Choose the first day of the week.', 'bfwc' ),
						'id'				=> 'bfwc_first_day',
						'default'		=> '0',
						'options'		=> array(
							'0'				=> __( 'Sunday', 'bfwc' ),
							'1'				=> __( 'Monday', 'bfwc' ),
							'2'				=> __( 'Tuesday', 'bfwc' ),
							'3'				=> __( 'Wednesday', 'bfwc' ),
							'4'				=> __( 'Thursday', 'bfwc' ),
							'5'				=> __( 'Friday', 'bfwc' ),
							'6'				=> __( 'Saturday', 'bfwc' )
						)
					),
					'bfwc_view_booked_days' => array(
						'name'			=> __( 'View Booked Days post type', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'		=> true,
						'desc'			=> __( 'For advanced use only: enable this to make the Booked Days post type public.', 'bfwc' ),
						'id'			=> 'bfwc_view_booked_days',
						'default'		=> 'no',
						'std'			=> 'no'
					),
					'section_end_calendar' => array(
						'type' => 'sectionend',
						'id' => 'bfwc_settings_title_calendar'
					),
				);

			} else if( $current_section == 'bfwc_search' ) {

				$settings = array(
					'section_title_search' => array(
						'name'     => __( 'Search', 'bfwc' ),
						'type'     => 'title',
						'desc'     => '',
						'id'       => 'bfwc_settings_title_search'
					),
					'bfwc_archive_search' => array(
						'name'			=> __( 'Enable search on archive pages', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'		=> true,
						'desc'			=> __( 'Enable this to display a search field on the shop and archive pages.', 'bfwc' ),
						'id'			=> 'bfwc_archive_search',
						'default'		=> '',
					),
					'section_end_search' => array(
						'type' => 'sectionend',
						'id' => 'bfwc_settings_title_search'
					),
				);

			} else if( $current_section == 'bfwc_external' ) {

				$settings = array(
					'section_title_external' => array(
						'name'     => __( 'External services', 'bfwc' ),
						'type'     => 'title',
						'desc'     => '',
						'id'       => 'bfwc_settings_title_external'
					),
					'bfwc_exclude_last_day' => array(
						'name'			=> __( 'Exclude last day', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'	=> true,
						'desc'			=> __( 'Enable this to ignore the last day of bookings imported from third party sites.', 'bfwc' ),
						'id'				=> 'bfwc_exclude_last_day',
						'default'		=> 'yes',
					),
					'bfwc_auto_sync' => array(
						'name'			=> __( 'Auto-sync calendars', 'bfwc' ),
						'type'			=> 'checkbox',
						'desc_tip'	=> true,
						'desc'			=> __( 'Enable this to automatically sync with calendars from external sites like Airbnb.', 'bfwc' ),
						'id'				=> 'bfwc_auto_sync',
						'default'		=> 'no',
					),
					'bfwc_sync_interval' => array(
						'name'			=> __( 'Sync interval', 'bfwc' ),
						'type'			=> 'number',
						'desc_tip'		=> true,
						'desc'			=> __( 'Interval in minutes to auto sync. Syncing too often might affect your site performance', 'bfwc' ),
						'id'			=> 'bfwc_sync_interval',
						'default'		=> 60,
						'custom_attributes'	=> array(
							'min'		=> 30,
							'step'		=> 1
						)
					),
					'section_end_external' => array(
						'type' => 'sectionend',
						'id' => 'bfwc_settings_title_external'
					),
				);

			} else if( $current_section == 'bfwc_lk' ) {

				$settings = array(
					'section_title' => array(
						'name'     => __( 'Licence', 'bfwc' ),
						'type'     => 'title',
						'desc'     => '',
						'id'       => 'bfwc_settings_title'
					),
					'bfwc_license_key' => array(
						'name'		=> __( 'license key', 'bfwc' ),
						'type'		=> 'bfwc_license_key',
						'desc_tip'	=> true,
						'desc'		=> __( 'Enter your license key here. You should have received a key with the email containing the plugin download link.', 'bfwc' ),
						'id'		=> 'bfwc_license_key',
						'default'	=> '',
					),
					'section_end' => array(
						'type' => 'sectionend',
						'id' => 'bfwc_settings_title'
					),

				);

			}

			return apply_filters( 'bfwc_filter_settings', $settings );

		}

		/**
		 * Output sections.
		 */
		public function output_sections() {

			global $current_section;
			echo '<ul class="subsubsub">';

				// Main settings tab
				$sections = array(
					'bfwc'			=> __( 'General', 'bfwc' ),
					'bfwc_calendar'	=> __( 'Calendar', 'bfwc' ),
					'bfwc_search'	=> __( 'Search', 'bfwc' ),					
					'bfwc_external'	=> __( 'External', 'bfwc' )
				);

				$sections['bfwc_lk'] = __( 'Licence', 'bfwc' );

				$sections = apply_filters( 'bfwc_settings_sections', $sections );

				$array_keys = array_keys( $sections );

				foreach( $sections as $id=>$label ) {
					echo '<li><a href="' . admin_url( 'admin.php?page=wc-settings&tab=bookings&section=' . sanitize_title( $id ) ) . '" class="' . ( ( $current_section == $id || ( ! $current_section && $id == 'bfwc' ) ) ? 'current' : '' ) . '">' . $label . '</a> ' . ( end( $array_keys ) == $id ? '' : '|' ) . ' </li>';
				}

			echo '</ul><br class="clear" />';

		}

		/**
		 * Custom setting for EDD SL licence key
		 */
		public function license_key() {
			$key = get_option( 'bfwc_license_key' ); ?>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<?php _e( 'Licence key', 'bfwc' ); ?>
				</th>
				<td class="forminp forminp-text">
					<input name="bfwc_license_key" id="bfwc_license_key" type="text" style="" value="<?php echo $key; ?>" class="" placeholder="">
				</td>
			</tr>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<?php _e( 'Status', 'bfwc' ); ?>
				</th>
				<td class="forminp forminp-text">
					<?php $status = ( false !== get_option( 'bfwc_license_status' ) ) ? get_option( 'bfwc_license_status' ) : 'invalid';
					if( $status == 'valid' ) {
						echo '<span class="dashicons dashicons-yes"></span>&nbsp;';
						echo ucfirst( $status );
					} else {
						// echo '<span class="dashicons dashicons-no-alt"></span>&nbsp;';
						if( ! $key ) {
							_e( 'Please enter your licence key', 'bfwc' );
						} else if( $status == 'invalid' ) {
							// $error = get_option( 'bfwc_test_response', false );
							printf(
								"<p>%s</p>",
							 	__( "It hasn't been possible to validate your licence key. Please check that you've entered it correctly. Remember, the licence key will be on the email you were sent with the downloadable plugin files.", 'bfwc' )
							);
						}
					} ?>
				</td>
				<?php if( $status == 'valid' ) { ?>
					<tr>
						<th scope="row" class="titledesc">
							<?php _e( 'Action', 'bfwc' ); ?>
						</th>
						<td class="forminp forminp-text">
							<?php printf(
								'<p><button type="submit" name="bfwc_deactivate_license_key" class="button button-secondary">%s</button></p>',
								__( 'Deactivate this licence', 'bfwc' )
							); ?>
						</td>
					</tr>
				<?php } else if( $status == 'deactivated' ) { ?>
					<tr>
						<th scope="row" class="titledesc">
							<?php _e( 'Action', 'bfwc' ); ?>
						</th>
						<td class="forminp forminp-text">
							<?php printf(
								'<p><button type="submit" name="bfwc_activate_license_key" class="button button-secondary">%s</button></p>',
								__( 'Activate this licence', 'bfwc' )
							); ?>
						</td>
					</tr>
				<?php } ?>
			</tr>
			<?php
			wp_nonce_field( 'bfwc_license_key_nonce', 'bfwc_license_key_nonce' );
		}

		public function role_price_fields() {
			$role_price_fields = bfwc_get_role_price_fields(); ?>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<?php _e( 'Roles', 'wcfad' ); ?>
				</th>
				<td class="forminp forminp-text">
					<select class="bfwc_select2" multiple="multiple" name="bfwc_role_price_fields[]">
						<?php $roles = bfwc_get_user_roles();
						if( $roles ) {
							foreach( $roles as $id=>$name ) {
								$selected = in_array( $id, $role_price_fields ) ? 'selected' : '';
								printf(
									'<option value="%s" %s>%s</option>',
									$id,
									$selected,
									$name
								);
							}
						} ?>
					</select>
				</td>
			</tr>
		<?php }

		/**
		 * Sanitise the payment methods field
		 * Ensure we pass back an empty array rather than just a null field
		 */
		public function sanitize_role_price_fields( $value, $option, $raw_value ) {

			if( ! $value ) {
				$value = array();
			}
			return $value;

		}

	}
	$BFWC_Booking_Settings_Tab = new BFWC_Booking_Settings_Tab;
	$BFWC_Booking_Settings_Tab->init();
}
