<?php
/**
 * Custom email to send before a booking starts
 * @since 2.0.0
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WC_Before_Start_Booking_Email extends WC_Email {

  private $booking_id;
  private $date;

  public function __construct() {

    $this->id = 'wc_before_start_booking';
    $this->title = __( 'Before Booking Starts', 'bfwc' );
    $this->description = __( 'Emails sent a set period before a booking starts', 'bfwc' );
    $this->heading = __( 'Your booking is due to start soon', 'bfwc' );;
    $this->subject = __( 'Your booking is due to start soon', 'bfwc' );;
    $this->template_html  = 'emails/before-booking-starts.php';
    $this->template_plain = 'emails/before-booking-starts.php';
		$this->template_base  = trailingslashit( BFWC_PLUGIN_DIR_PATH ) . 'admin/templates/';

    $this->booking_id = null;
    $this->date = null;

    // Trigger when form is sent
    add_action( 'bfwc_trigger_before_booking_starts_email', array( $this, 'trigger' ), 10, 2 );

    parent::__construct();

  }

  public function trigger( $booking_id, $date ) {

		$this->booking_id = $booking_id;
		$this->date = $date;
    $this->recipient = get_post_meta( $booking_id, 'bfwc_user_email', true );

    if ( ! $this->is_enabled() || ! $this->get_recipient() ) {
      return;
    }
    $this->send( $this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments() );

  }

  public function get_content_html() {
    return wc_get_template_html(
        $this->template_html,
            array(
              'booking_id'    => $this->booking_id,
              'date'          => $this->date,
              'email_heading' => $this->get_heading(),
              'additional_content' => $this->get_additional_content(),
              'sent_to_admin' => false,
              'plain_text'    => false,
              'email'			    => $this
            ),
        '',
        $this->template_base );
    }

    /**
     * get_content_plain function.
     *
     * @since 0.1
     * @return string
     */
    public function get_content_plain() {
      return wc_get_template_html(
        $this->template_plain,
        array(
          'booking_id'    => $this->booking_id,
          'date'          => $this->date,
          'email_heading' => $this->get_heading(),
          'additional_content' => $this->get_additional_content(),
          'sent_to_admin' => false,
          'plain_text'    => true,
          'email'			=> $this
        ),
        '',
        $this->template_base
      );
    }

    public function init_form_fields() {

      $this->form_fields = array(
        'enabled'    => array(
          'title'   => 'Enable/Disable',
          'type'    => 'checkbox',
          'label'   => 'Enable this email notification',
          'default' => 'no'
        ),
        // 'recipient'  => array(
        //   'title'       => 'Recipient(s)',
        //   'type'        => 'text',
        //   'description' => sprintf( 'Enter recipients (comma separated) for this email. Defaults to <code>%s</code>.', esc_attr( get_option( 'admin_email' ) ) ),
        //   'placeholder' => '',
        //   'default'     => ''
        // ),
        'subject'    => array(
          'title'       => 'Subject',
          'type'        => 'text',
          'description' => sprintf( 
            __( 'This controls the email subject line. Leave blank to use the default subject: <code>%s</code>.', 'bfwc' ),
            $this->subject ),
          'placeholder' => '',
          'default'     => __( 'Your booking is due to start soon', 'bfwc' )
        ),
        'heading'    => array(
          'title'       => 'Email Heading',
          'type'        => 'text',
          'description' => sprintf( 
            __( 'This controls the main heading contained within the email notification. Leave blank to use the default heading: <code>%s</code>.', 'bfwc' ),
            $this->heading ),
          'placeholder' => '',
          'default'     => __( 'Your booking is due to start soon', 'bfwc' )
        ),
        'additional_content' => array(
					'title'       => __( 'Content', 'bfwc' ),
					'description' => __( 'Text to appear below the main email content.', 'bfwc' ),
					'css'         => 'width:400px; height: 75px;',
					// 'placeholder' => sprintf(
          //   __( 'Your booking %s is due to start on %s', 'bfwc' ),
          //   $this->booking_id,
          //   $this->date
          // ),
					'type'        => 'textarea',
					'default'     => sprintf(
            __( 'Your booking {booking_id} is due to start on {date}', 'bfwc' )
          ),
					'desc_tip'    => true,
				),
        'email_type' => array(
          'title'       => 'Email type',
          'type'        => 'select',
          'description' => 'Choose which format of email to send.',
          'default'     => 'html',
          'class'       => 'email_type',
          'options'     => array(
              'plain'     => 'Plain text',
              'html'      => 'HTML', 'bfwc',
              'multipart' => 'Multipart', 'bfwc',
          )
        ),
        'days'  => array(
          'title'       => 'Days',
          'type'        => 'number',
          'description' => __( 'Number of days before booking starts to send this email', 'bfwc' ),
          'placeholder' => '',
          'default'     => ''
        ),
    );
  }

}
