<?php
/**
 * Class to create our new booking product type
 * @since	1.0.0
 * @version 2.0.8
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

// Ensure WooCommerce has loaded
if( ! class_exists( 'WC_Product' ) ) {
	return;
}

if( ! class_exists( 'WC_Product_Simple_Booking' ) ) {
	
	class WC_Product_Simple_Booking extends WC_Product {

		private $aelia_price;
		private $aelia_currency = false;

		public function __construct( $product ) {

			// 2.0.5, both cause PHP deprecated warnings in PHP 8.2
			//$this->product_type = 'simple_booking'; // 2.0.5, the product_type property is deprecated. We now use get_type(), but requires WooCommerce 3.0.0
			//$this->virtual = true;

			parent::__construct( $product );
			$this->set_virtual( true ); // 2.0.5

		}
		
		/**
		 * Get price HTML
		 * Filtered in functions-single-product.php
		 * @param string $price
		 * @return string
		 */
		public function get_price_html( $price = '' ) {
			if ( '' === $this->get_price() ) {
				$price = apply_filters( 'woocommerce_empty_price_html', '', $this );
			} elseif ( $this->is_on_sale() ) {
				$price = wc_format_sale_price( wc_get_price_to_display( $this, array( 'price' => $this->get_regular_price() ) ), wc_get_price_to_display( $this ) ) . $this->get_price_suffix();
			} else {
				$price = wc_price( wc_get_price_to_display( $this ) ) . $this->get_price_suffix();
			}			
			return apply_filters( 'woocommerce_get_price_html', $price, $this );
		}
		
		/**
		 * Bookings are always purchasable
		 */
		public function is_purchasable() {
			$status = is_callable( array( $this, 'get_status' ) ) ? $this->get_status() : $this->post->post_status;
			return apply_filters( 'woocommerce_is_purchasable', $this->exists() && ( 'publish' === $status || current_user_can( 'edit_post', $this->get_id() ) ), $this );
		}
		
		/**
		 * Get the standard cost per unit.
		 *
		 * @param string $context
		 * @return string
		 */
		public function get_price( $context = 'view' ) {
			$price = parent::get_price( $context );

			if ( ! $price ) {
				$price = bfwc_get_standard_cost( $this );
			}

			return apply_filters( 'woocommerce_product_get_price', $price, $this );
		}

		/**
		 * Set product type, requires WooCommerce 3.0.0
		 * @since 2.0.5
		 */
		public function get_type() {
			return 'simple_booking';
		}

		/**
		 * Save a product's converted price
		 * @since 2.0.7
		 */
		public function set_aelia_price( $currency, $price ) {
			$this->aelia_price[$currency] = $price;
			// also set the currency here?
			$this->set_aelia_currency( $currency );
		}

		/**
		 * Get a product's converted price
		 * @since 2.0.7
		 */
		public function get_aelia_price( $currency ) {
			if ( isset( $this->aelia_price[$currency] ) ) {
				return $this->aelia_price[$currency];
			} else {
				return false;
			}
		}

		/**
		 * Save the product's current currency
		 * @since 2.0.8
		 */
		public function set_aelia_currency( $currency ) {
			$this->aelia_currency = $currency;
		}

		/**
		 * Get the product's current currency
		 * @since 2.0.8
		 */
		public function get_aelia_currency() {
			if ( false === $this->aelia_currency ) {
				$this->aelia_currency = get_option( 'woocommerce_currency' ); // get the default
			}
			return $this->aelia_currency;
		}
	}
	
}