<?php
/**
 * Functions for integrating with various plugins
 * @since 2.0.5
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Compatibility with Aelia Currency Switcher with WooCommerce
 * @since	2.0.5
 * @version 2.0.7
 */
function bfwc_aelia_get_price( $price, $product ) {
	if( bfwc_has_aelia() && $product && 'simple_booking' === $product->get_type() && ! empty( $price ) ) {
		// Aelia is installed
		$from_currency = $product->get_aelia_currency();
		$to_currency = WC_Aelia_CurrencySwitcher::instance()->get_selected_currency();

		if ( '' != $from_currency && '' != $to_currency ) {
			// 2.0.7, check if the price was converted already
			$bfwc_price = $product->get_aelia_price( $to_currency );
			if ( false === $bfwc_price ) {
				// not yet converted, convert
				$price = apply_filters( 'wc_aelia_cs_convert', $price, $from_currency, $to_currency );
				// save the converted price with the currency
				$product->set_aelia_price( $to_currency, $price );
			} else {
				$price = $bfwc_price;
			}
		}
	}
	return $price;
}
add_filter( 'woocommerce_product_get_price', 'bfwc_aelia_get_price', 10, 2 );

/**
 * Check if Aelia is activated
 * @since 2.0.8
 */
function bfwc_has_aelia() {
	return class_exists( 'WC_Aelia_CurrencySwitcher' );
}

/**
 * Save selected currency for the price added to the cart
 * @since 2.0.8
 */
function bwfc_aelia_add_cart_item_data( $cart_item_meta, $product_id ) {
	if ( ! bfwc_has_aelia() ) {
		return $cart_item_meta;
	}

	$product = wc_get_product( $product_id );

	// If this isn't a booking, we can leave at this point
	if( ! $product->is_type( 'simple_booking' ) ) {
		return $cart_item_meta;
	}

	$cart_item_meta['booking']['aelia_currency'] = WC_Aelia_CurrencySwitcher::instance()->get_selected_currency();

	return $cart_item_meta;
}
add_filter( 'woocommerce_add_cart_item_data', 'bwfc_aelia_add_cart_item_data', 11, 2 );

/**
 * Prevents double conversion in the cart
 * @since 2.0.8
 */
function bfwc_aelia_booking_item( $cart_object ) {
	if ( bfwc_has_aelia() ) {
		foreach ( $cart_object->get_cart() as $key=>$value ) {
			if ( ! empty ( $value['booking']['aelia_currency'] ) && ! $value['data']->get_aelia_price( $value['booking']['aelia_currency'] ) ) {
				$value['data']->set_aelia_price( $value['booking']['aelia_currency'], $value['booking']['cost'] ); // prevents double conversion of booking products
			}
		}
	}
	return $cart_object;
}
add_action( 'woocommerce_before_calculate_totals', 'bfwc_aelia_booking_item', 11 );
