<?php
/**
 * Functions for booking prices
 * @since 2.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Return an array of prices for booking products per day
 * This is used in the title attribute of the calendar cells
 * @since 2.0.0
 */
function bfwc_get_prices_per_day( $product_id=false ) {

  if( ! $product_id ) {
		$product_id = get_the_ID();
	}

	// Only show a daily price for day and night units, not weekly
	$unit = bfwc_get_booking_unit( $product_id );
	if( $unit == 'week' ) {
		return false;
	}

	$product = wc_get_product( $product_id );

	$prices_by_day = array();

	// This is our list of non standard costs
	$nonstd_data = get_post_meta( $product_id, '_nonstd_data', true );

	if( $nonstd_data ) {

		// Organise it into an array
		$nonstd_dates = bfwc_get_nonstd_dates_array( $nonstd_data );

		// Build up our array of prices
		if( ! empty( $nonstd_dates['costs'] ) ) {

			foreach( $nonstd_dates['costs'] as $date=>$cost ) {

				if ( 'incl' === get_option( 'woocommerce_tax_display_shop' ) ) {
					$price = wc_get_price_including_tax( $product, array( 'price' => $cost ) );
				} else {
					$price = wc_get_price_excluding_tax( $product, array( 'price' => $cost ) );
				}

				// Format the price
				$prices_by_day[$date] = html_entity_decode( strip_tags( wc_price( $price) ) );

			}

		}

	}

  return $prices_by_day;

}
