<?php
/**
 * Functions for display Booking elements in single-product.php
 * @since 1.0.0
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Returns whether product is bookable
 * @return Boolean
 */
function bfwc_is_simple_booking_product( $product_id='' ) {
	if( ! $product_id ) {
		$product_id = get_the_ID();
	}
	if( 'product' != get_post_type( $product_id ) ) {
		return false;
	}
	$product = wc_get_product( $product_id );
	if( $product->is_type( 'simple_booking' ) ) {
		return true;
	}
	return false;
}

/**
 * Add booking custom fields inside the add-to-cart form
 */
function bfwc_additional_fields() {
	if( bfwc_is_simple_booking_product() ) {
		$view = 'double'; ?>
		<div class="booking-form-inner">
			<div class="booking-fields">
				<?php do_action( 'bfwc_booking_form_' . esc_attr( $view ) ); ?>
		    </div>
			<input type="hidden" id="product_id" name="product_id" value="<?php echo esc_attr( get_the_ID() ); ?>" />
			<input type="hidden" id="pending_dates" name="pending_dates" value="" />
			<input type="hidden" id="start_date_ymd" name="start_date_ymd" value="" />
			<input type="hidden" id="end_date_ymd" name="end_date_ymd" value="" />
			<input type="hidden" id="calculated_booking_cost" name="calculated_booking_cost" value="" />
			<input type="hidden" id="num_units_int" name="num_units_int" value="" />
			<input type="hidden" id="permalink" name="permalink" value="<?php echo esc_attr( get_permalink() ); ?>" />
			<?php wp_nonce_field( 'bookings_nonce', 'bookings_nonce' ); ?>
			<div class="loader-background"></div>
			<div class="loader"></div>
		</div><!-- .booking-form-inner -->
	<?php }
	}
add_action( 'woocommerce_before_add_to_cart_button', 'bfwc_additional_fields' );

/**
 * The mark up for the single calendar view
 */
function bfws_single_calendar_view() {
	if( bfwc_is_simple_booking_product() ) {
		$label = bfwc_get_unit_label( get_the_ID() ); ?>
	    <div id="bfws-calendar"></div>
		<div class="bfws-booking-details">
			<table>
				<colgroup>
					<col width="40%">
					<col width="20%">
					<col width="40%">
				</colgroup>
				<tr><th><?php _e( 'Start', 'bfws' ); ?></th><th><?php echo esc_html( $label['plural'] ); ?></th><th><?php _e( 'End', 'bfws' ); ?></th></tr>
				<tr><td class="booking-details-start-date"></td><td><input type="number" min="1" id="num_units" name="num_units" value="" /></td><td class="booking-details-end-date"></td></tr>
				<tr><td class="booking-cost-header" colspan="2"><?php _e( 'Booking cost:', 'bfwc' ); ?></td><td class="booking-cost"></td></tr>
			</table>
			<input type="hidden" id="start_date" name="start_date" value="" />
		</div>
	<?php }
}
add_action( 'bfwc_booking_form_single', 'bfws_single_calendar_view' );

/**
 * The mark up for the double calendar view
 */
function bfws_double_calendar_view() {
	if( bfwc_is_simple_booking_product() ) {

		$auto_set_end = bfwc_get_auto_set_end( get_the_ID() );
		if( $auto_set_end != 'yes' ) {
			$colspan = 1;
		} else {
			$colspan = 2;
		}

		$product_id = get_the_ID(); ?>
	    <div id="bfws-calendar-wrapper">
			<table>
				<tbody>
					<tr>
						<td colspan="<?php echo $colspan; ?>" class="bfws-date-field-wrapper">
							<i class="gg-calendar-dates"></i>
							<input type="text" class="bfws-start-date" id="start_date" name="start_date" value="" placeholder="<?php echo apply_filters( 'bfwc_filter_check_in_placeholder', __( 'Check in', 'bfwc' ), $product_id ); ?>" readonly="readonly" />
							<?php if( $auto_set_end == 'yes' ) { ?>
								<input type="hidden" class="bfws-end-date" id="end_date" name="end_date" value="" placeholder="<?php echo apply_filters( 'bfwc_filter_check_out_placeholder', __( 'Check out', 'bfwc' ), $product_id ); ?>" readonly="readonly" />
							<?php } ?>
						</td>
						<?php if( $auto_set_end != 'yes' ) { ?>
							<td class="bfws-date-field-wrapper"><i class="gg-calendar-dates"></i><input type="text" class="bfws-end-date" id="end_date" name="end_date" value="" placeholder="<?php echo apply_filters( 'bfwc_filter_check_out_placeholder', __( 'Check out', 'bfwc' ), $product_id ); ?>" readonly="readonly" /></td>
						<?php } ?>
					</tr>
				</tbody>
				<tfoot>
					<?php
					if( bfwc_get_max_bookings( $product_id ) > 1 ) { ?>
						<tr>
							<td class="booking-cost-header">
								<?php echo apply_filters( 'bfwc_quantity_bookings_text', __( 'Quantity:', 'bfwc' ), $product_id ); ?></td>
							<td class="booking-cost-data booking-cost-data-number">
								<?php printf(
									'<input type="number" name="num_bookings" id="num_bookings" step="1" min="1" max="%s" value="%s">',
									bfwc_get_max_bookings( $product_id ),
									apply_filters( 'bfwc_default_num_bookings', 1, $product_id )
								); ?>
							</td>
						</tr>
					<?php } ?>
					<tr>
						<td class="booking-cost-header"><?php _e( 'Duration:', 'bfwc' ); ?></td><td id="num_units" class="booking-cost-data"></td>
					</tr>
					<tr>
						<td class="booking-cost-header"><?php _e( 'Cost:', 'bfwc' ); ?></td><td class="booking-cost-data booking-cost"></td>
					</tr>
				</tfoot>
			</table>
	    </div>
	<?php }
}
add_action( 'bfwc_booking_form_double', 'bfws_double_calendar_view' );

/**
 * Print a nice table for booking costs
 * Hook this to woocommerce_after_add_to_cart_form or similar
 */
function bfws_pricing_table() {
	if( bfwc_is_simple_booking_product() ) {
		$label = bfwc_get_unit_label( get_the_ID() );
		$post_id = get_the_ID();
		$product = wc_get_product( $post_id );
		$std_cost = bfwc_get_standard_cost( $product );
		$nonstd_data = get_post_meta( $post_id, '_nonstd_data', true ); ?>
		<table class="pricing-table">
			<thead>
				<tr>
					<td><?php echo apply_filters( 'bfws_filter_pricing_table_heading', __( 'Pricing Details', 'bfwc' ) ); ?></td>
					<td class="pricing-table-cost"><?php echo apply_filters( 'bfws_filter_pricing_table_unit', __( 'Per ', 'bfwc' ) . strtolower( $label['single'] ), $label ); ?></td>
				</tr>
			</thead>
			<tbody>
				<tr>
					<td><?php echo apply_filters( 'bfws_filter_pricing_table_standard', __( 'Standard Cost', 'bfwc' ) ); ?></td>
					<td class="pricing-table-cost"><?php echo get_woocommerce_currency_symbol() . $std_cost; ?></td>
				</tr>
				<?php if( $nonstd_data ) {
					$date_format = get_option( 'date_format' );
					foreach( $nonstd_data as $item ) {
						if( isset( $item['start_date'] ) && isset( $item['end_date'] ) & isset( $item['cost'] ) && $item['cost'] > 0 ) {
							$start = new DateTime( $item['start_date'] );
							// $num_units = $item['num_units'];
							$end_date = new DateTime( $item['end_date'] ) ?>
							<tr>
								<td><?php echo $item['start_date'] . ' - ' . $item['end_date']; ?></td>
								<td class="pricing-table-cost"><?php echo get_woocommerce_currency_symbol() . $item['cost']; ?></td>
							</tr>
						<?php }
					}
				} ?>
			</tbody>
		</table>
	<?php }
}

/**
 * Remove quantity input field on single product page for simple_booking product type
 */
function bfwc_remove_quantity_input_single( $return, $product ) {
	if( $product->is_type( 'simple_booking' ) ) {
		return true;
	}
	return $return;
}
add_filter( 'woocommerce_is_sold_individually', 'bfwc_remove_quantity_input_single', 10, 2 );

/**
 * Filter the price html to show cost per unit
 */
function bfwc_filter_price_html( $price, $product ) {
	if( $product->is_type( 'simple_booking' ) ) {
		// Check if the product has custom prices
		$nonstd_data = get_post_meta( $product->get_ID(), '_nonstd_data', true );
		if( $nonstd_data ) {
			$price = __( 'From', 'bfwc' ) . ' ' . $price;
		}
		$label = bfwc_get_unit_label( $product->get_ID() );
		$price .= ' ' . __( 'per', 'bfwc' ) . ' ' . esc_html( strtolower( $label['single'] ) );
		return $price;
	}
	return $price;
}
add_filter( 'woocommerce_get_price_html', 'bfwc_filter_price_html', 10, 2 );

/**
 * Get the label for the unit we're using
 * @param $post_id
 */
function bfwc_get_unit_label( $post_id=null ) {
	$labels = array(
		'day'	=> array(
			'single' => __( 'Day', 'bfwc' ),
			'plural' => __( 'Days', 'bfwc' ),
		),
		'night'	=> array(
			'single' => __( 'Night', 'bfwc' ),
			'plural' => __( 'Nights', 'bfwc' ),
		),
		'week'	=> array(
			'single' => __( 'Week', 'bfwc' ),
			'plural' => __( 'Weeks', 'bfwc' ),
		),
		'unit'	=> array(
			'single' => __( 'Unit', 'bfwc' ),
			'plural' => __( 'Units', 'bfwc' ),
		),
	);
	$labels = apply_filters( 'bfwc_filter_unit_labels', $labels, $post_id );
	if( ! $post_id ) {
		$key = 'unit';
	} else {
		$key = bfwc_get_booking_unit( $post_id );
	}
	if( isset( $labels[$key] ) ) {
		return $labels[$key];
	}
	return false;
}
