<?php
/**
 * Functions for updating the plugin
 * @since 1.0.3
 * @package Bookings for WooCommerce
 */

// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}

$store_url = get_option( 'bfwc_store_url', 'https://pluginrepublic.com' );
define( 'BFWC_STORE_URL', 'https://pluginrepublic.com' );

// the name of your product. This should match the download name in EDD exactly
define( 'BFWC_ITEM_NAME', 'Bookings for WooCommerce' ); // you should use your own CONSTANT name, and be sure to replace it throughout this file

define( 'BFWC_ITEM_ID', 3602 );

if( ! class_exists( 'BFWC_SL_Plugin_Updater' ) ) {
	// load our custom updater
	include( BFWC_DIRNAME . '/classes/class-bfwc-sl-plugin-updater.php' );
}

/**
 * Every day, check to see if we've migrated to Plugin Republic
 */
function bfwc_daily_check_store_url() {
	// $store_url = get_option( 'bfwc_store_url', false );
	// if( $store_url != 'https://pluginrepublic.com' ) {
	// 	// Still haven't migrated, so check
	// 	$file_headers = @get_headers( "https://pluginrepublic.com/downloads/product-extras-woocommerce-product-addons/" );
	// 	if( isset( $file_headers[0] ) && strpos( $file_headers[0], '200' ) !== false ) {
	// 		update_option( 'bfwc_store_url', 'https://pluginrepublic.com' );
	// 	} else {
	// 		update_option( 'bfwc_store_url', 'https://catapultthemes.com' );
	// 	}
	// }
}
// add_action( 'woocommerce_scheduled_sales', 'bfwc_daily_check_store_url' );

function bfwc_plugin_updater() {

	// retrieve our license key from the DB
	$license_key = trim( get_option( 'bfwc_license_key' ) );

	// setup the updater
	$edd_updater = new BFWC_SL_Plugin_Updater( BFWC_STORE_URL, BFWC_FILE, array(
			'version' 	=> BFWC_PLUGIN_VERSION, 				// current version number
			'license' 	=> $license_key, 		// license key (used get_option above to retrieve from DB)
			'item_name' => BFWC_ITEM_NAME, 	// name of this plugin
			'item_id' 	=> BFWC_ITEM_ID, 
			'author' 	=> 'Plugin Republic',  // author of this plugin
			'beta'		=> false
		)
	);

}
add_action( 'admin_init', 'bfwc_plugin_updater', 0 );

/**
 * Activate the licence key from the Submissions tab in WooCommerce > Settings
 */
function bfwc_activate_license() {

	// Check we're in WooCommerce settings
	if( ( isset( $_GET['page'] ) && $_GET['page'] == 'wc-settings' ) ) {

		// retrieve the license from the database
		$license = trim( get_option( 'bfwc_license_key' ) );

		// data to send in our API request
		$api_params = array(
			'edd_action' => 'activate_license',
			'license'    => $license,
			'item_name'  => urlencode( BFWC_ITEM_NAME ), // the name of our product in EDD
			'item_id' 	=> BFWC_ITEM_ID,
			'url'        => home_url()
		);

		// Call the custom API.
		$response = wp_remote_post( BFWC_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay
		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {

			if ( is_wp_error( $response ) ) {
				$message = $response->get_error_message();
			} else {
				// $message = __( 'An error occurred, please try again.', 'bfwc' );
			}

		} else {

			$license_data = json_decode( wp_remote_retrieve_body( $response ) );

			if ( false === $license_data->success ) {

				switch( $license_data->error ) {

					case 'expired' :

						$message = sprintf(
							__( 'Your license key expired on %s.', 'bfwc' ),
							date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
						);
						break;

					case 'revoked' :

						$message = __( 'Your license key has been disabled.', 'bfwc' );
						break;

					case 'missing' :

						$message = sprintf( __( 'Missing license key for %s. Please enter your license number to enable automatic updates and product support', 'bfwc' ), BFWC_ITEM_NAME );
						break;

					case 'invalid' :
					case 'site_inactive' :

						$message = __( 'Your license is not active for this URL.', 'bfwc' );
						break;

					case 'item_name_mismatch' :

						$message = sprintf( __( 'This appears to be an invalid license key for %s.', 'bfwc' ), BFWC_ITEM_NAME );
						break;

					case 'no_activations_left':

						$message = __( 'Your license key has reached its activation limit.', 'bfwc' );
						break;

					default :

						$message = '';
						break;
				}

			} else {

				$message = __( 'License activated', 'bfwc' );

			}

		}

		// $license_data->license will be either "valid" or "invalid"
		if( isset( $license_data->license ) ) {
			update_option( 'bfwc_license_status', $license_data->license );
		}
		update_option( 'bfwc_license_message', sanitize_text_field( $message ) );
		add_action( 'admin_notices', 'bfwc_license_validation_notice' );

	}
}
add_action('admin_init', 'bfwc_activate_license');


/**
 * Display license status message
 */
function bfwc_license_validation_notice() {
	$status = get_option( 'bfwc_license_status' );
	$message = get_option( 'bfwc_license_message' );
	if( empty( $_GET['tab'] ) || $_GET['tab'] != 'bookings' ) {
		return;
	}
	if( empty( $message ) ) return;
	if( $status == 'invalid' ) {
		update_option( 'bfwc_license_message_displayed', 0 ); ?>
		<div class="notice notice-error">
			<p><?php echo $message; ?></p>
		</div>
	<?php } else {
		$message_displayed = get_option( 'bfwc_license_message_displayed', 0 );
		if( empty( $_GET['section'] ) || $_GET['section'] != 'bfwc_lk' ) {
			return;
		}
		if( ! $message_displayed ) {
			// Only show success notice on Bookings tab once license is activated ?>
			<div class="notice notice-success">
				<p><?php echo $message; ?></p>
			</div>
		<?php update_option( 'bfwc_license_message_displayed', 1 );
	} else {
			// Show message that license is active and valid ?>
			<div class="notice notice-success">
				<p><?php _e( 'Your Bookings for WooCommerce license is valid and activated', 'bfwc' ); ?></p>
			</div>
		<?php }
	}
}
